<?php
if ( !defined( 'ABSPATH' ) ) {
	exit;
}

class FES_Install {

	public $new_settings = array();

	public function init() {
		$version = get_option( 'fes_current_version', '2.0' );

		if ( version_compare( $version, '2.3.2', '>=' )  ) {
			return;
		}
		$this->new_settings = get_option( 'fes_settings' );

		$version = get_option( 'fes_current_version', false );

		// if new install
		if ( !$version ) {
			$this->fes_new_install();
			update_option( 'fes_db_version', '2.3' );
		} else {
			if ( version_compare( $version, '2.1', '<' ) ) {
				$this->fes_v21_upgrades();
			}

			if ( version_compare( $version, '2.2', '<' ) ) {
				$this->fes_v22_upgrades();
			}

			if ( version_compare( $version, '2.3', '<' ) ) {
				$this->fes_v23_upgrades();
			}

			if ( version_compare( $version, '2.3.2', '<' ) ) {
				$this->add_new_roles();
			}
		}
		update_option( 'fes_current_version', '2.3.2' );
		update_option( 'fes_settings', $this->new_settings );
		set_transient( '_fes_activation_redirect', true, 30 );
		wp_redirect( admin_url() );
		exit;
	}

	public function fes_new_install() {
		$this->add_new_roles();
		$this->create_vendor_dashboard_page();
		$this->create_vendor_page();
		$this->create_submission_form();
		$this->create_profile_form();
		$this->create_registration_form();
		$this->create_login_form();
		$this->create_vendor_contact_form();
		$db = new FES_DB_Vendors();
		$db->create_table();
	}

	public function fes_v21_upgrades() {
		// No longer needed. None of the upgrade routine is used anymore.
	}

	public function fes_v22_upgrades() {
		// convert settings panel
		$old_settings = get_option( 'edd_fes_options', false );
		if ( $old_settings ) {
			// Submission form
			if ( isset( $old_settings['fes-submission-form'] ) && $old_settings['fes-submission-form'] != '' ) {
				$this->new_settings['fes-submission-form'] = $old_settings['fes-submission-form'];
			}

			// Profile form
			if ( isset( $old_settings['fes-profile-form'] ) && $old_settings['fes-profile-form'] != '' ) {
				$this->new_settings['fes-profile-form'] = $old_settings['fes-profile-form'];
			}

			// Application form
			if ( isset( $old_settings['fes-application-form'] ) && $old_settings['fes-application-form'] != '' ) {
				$this->new_settings['fes-application-form'] = $old_settings['fes-application-form'];
			}

			// Vendor form
			if ( isset( $old_settings['vendor-page'] ) && $old_settings['vendor-page'] != '' ) {
				$this->new_settings['fes-vendor-page'] = $old_settings['vendor-page'];
			}

			// Vendor Dashboard form
			if ( isset( $old_settings['vendor-dashboard-page'] ) && $old_settings['vendor-dashboard-page'] != '' ) {
				$this->new_settings['fes-vendor-dashboard-page'] = $old_settings['vendor-dashboard-page'];
			}

			// Vendor Dashboard notification
			if ( isset( $old_settings['dashboard-page-template'] ) && $old_settings['dashboard-page-template'] !== '' ) {
				$this->new_settings['fes-dashboard-notification'] = $old_settings['dashboard-page-template'];
			}

			// Allow Vendors Backend Access
			if ( isset( $old_settings['vendors_bea'] ) && $old_settings['vendors_bea'] !== '' ) {
				$this->new_settings['fes-allow-backend-access'] = $old_settings['vendors_bea'];
			}

			// Show Vendor Registration
			if ( isset( $old_settings['show_vendor_registration'] ) && $old_settings['show_vendor_registration'] !== '' ) {
				$this->new_settings['show_vendor_registration'] = $old_settings['show_vendor_registration'];
			}

			// Auto Approve Vendors
			if ( isset( $old_settings['edd_fes_auto_approve_vendors'] ) && $old_settings['edd_fes_auto_approve_vendors'] !== '' ) {
				$this->new_settings['fes-auto-approve-vendors'] = $old_settings['edd_fes_auto_approve_vendors'];
			}

			// Allow Vendors to Edit Products
			if ( isset( $old_settings['edd_fes_vendor_permissions_edit_product'] ) && $old_settings['edd_fes_vendor_permissions_edit_product'] !== '' ) {
				$this->new_settings['fes-allow-vendors-to-edit-products'] = $old_settings['edd_fes_vendor_permissions_edit_product'];
			}

			// Allow Vendors to Delete Products
			if ( isset( $old_settings['edd_fes_vendor_permissions_delete_product'] ) && $old_settings['edd_fes_vendor_permissions_delete_product'] !== '' ) {
				$this->new_settings['fes-allow-vendors-to-delete-products'] = $old_settings['edd_fes_vendor_permissions_delete_product'];
			}

			// Use EDD's CSS
			if ( isset( $old_settings['edd_fes_use_css'] ) && $old_settings['edd_fes_use_css'] !== '' ) {
				$this->new_settings['fes-use-css'] = $old_settings['edd_fes_use_css'];
			}

			// Admin notification on new vendor application
			if ( isset( $old_settings['edd_fes_notify_admin_new_app_toggle'] ) && $old_settings['edd_fes_notify_admin_new_app_toggle'] !== '' ) {
				$this->new_settings['fes-admin-new-app-email-toggle'] = $old_settings['edd_fes_notify_admin_new_app_toggle'];
			}

			// Admin message on new vendor application
			if ( isset( $old_settings['edd_fes_notify_admin_new_app_message'] ) && $old_settings['edd_fes_notify_admin_new_app_message'] != '' ) {
				$this->new_settings['fes-admin-new-app-email'] = $old_settings['edd_fes_notify_admin_new_app_message'];
			}

			// User message on new vendor application
			if ( isset( $old_settings['edd_fes_notify_user_new_app_message'] ) && $old_settings['edd_fes_notify_user_new_app_message'] != '' ) {
				$this->new_settings['fes-vendor-new-app-email'] = $old_settings['edd_fes_notify_user_new_app_message'];
			}

			// Admin message on new vendor submission
			if ( isset( $old_settings['new_edd_fes_submission_admin_message'] ) && $old_settings['new_edd_fes_submission_admin_message'] != '' ) {
				$this->new_settings['fes-admin-new-submission-email'] = $old_settings['new_edd_fes_submission_admin_message'];
			}

			// User message on vendor application accepted
			if ( isset( $old_settings['edd_fes_notify_user_app_accepted_message'] ) && $old_settings['edd_fes_notify_user_app_accepted_message'] != '' ) {
				$this->new_settings['fes-vendor-app-approved-email'] = $old_settings['edd_fes_notify_user_app_accepted_message'];
			}

			// User message on vendor application denied
			if ( isset( $old_settings['edd_fes_notify_user_app_denied_message'] ) && $old_settings['edd_fes_notify_user_app_denied_message'] != '' ) {
				$this->new_settings['fes-vendor-app-declined-email'] = $old_settings['edd_fes_notify_user_app_denied_message'];
			}

			// User message on new vendor submission
			if ( isset( $old_settings['new_edd_fes_submission_user_message'] ) && $old_settings['new_edd_fes_submission_user_message'] != '' ) {
				$this->new_settings['fes-vendor-new-submission-email'] = $old_settings['new_edd_fes_submission_user_message'];
			}

			// User message on new vendor submission accepted
			if ( isset( $old_settings['edd_fes_submission_accepted_message'] ) && $old_settings['edd_fes_submission_accepted_message'] != '' ) {
				$this->new_settings['fes-vendor-submission-approved-email'] = $old_settings['edd_fes_submission_accepted_message'];
			}

			// User message on new vendor submission declined
			if ( isset( $old_settings['edd_fes_submission_declined_message'] ) && $old_settings['edd_fes_submission_declined_message'] != '' ) {
				$this->new_settings['fes-vendor-submission-declined-email'] = $old_settings['edd_fes_submission_declined_message'];
			}

			// reCAPTCHA Public Key
			if ( isset( $old_settings['recaptcha_public'] ) && $old_settings['recaptcha_public'] != '' ) {
				$this->new_settings['fes-recaptcha-public-key'] = $old_settings['recaptcha_public'];
			}

			// reCAPTCHA Private Key
			if ( isset( $old_settings['recaptcha_private'] ) && $old_settings['recaptcha_private'] != '' ) {
				$this->new_settings['fes-recaptcha-private-key'] = $old_settings['recaptcha_private'];
			}
		}
		$this->create_registration_form();
		$this->create_login_form();
		$this->create_vendor_contact_form();

		// if application form
		if ( isset( $this->new_settings['fes-application-form'] ) && $this->new_settings['fes-application-form'] != '' ) {
			// move fields to registration form
			$old_fields = get_post_meta( $this->new_settings['fes-application-form'], 'fes-form', true );
			$new_fields = get_post_meta( $this->new_settings['fes-registration-form'], 'fes-form', true );

			if ( is_array( $old_fields ) && is_array( $new_fields ) ) {
				$counter = 0;
				foreach ( $old_fields as $field ) {
					$key = 7 + $counter;
					if ( isset( $field['input_type'] ) && $field['input_type'] == 'image_upload' && isset( $field['is_meta'] ) && $field['is_meta'] == 'no' ) {
						$field['input_type'] = 'file_upload';
					}
					if ( isset( $field['template'] ) && $field['template'] == 'image_upload' && isset( $field['is_meta'] ) && $field['is_meta'] == 'no' ) {
						$field['template'] = 'file_upload';
					}
					// skip these fields as they are already in the new form
					$to_skip = array( 'last_name', 'first_name', 'user_email', 'username', 'password', 'display_name' );
					if ( isset( $field['template'] ) && !in_array( $field['template'] , $to_skip ) ) {
						$new_fields[$key] = $field;
						$counter++;
					}
				}
				update_post_meta( $this->new_settings['fes-registration-form'], 'fes-form', $new_fields );
			}
		}

		// if submission form
		if ( isset( $this->new_settings['fes-submission-form'] ) && $this->new_settings['fes-submission-form'] != '' ) {
			$old_fields = get_post_meta( $this->new_settings['fes-submission-form'], 'fes-form', true );
			if ( !is_array( $old_fields ) ) {
				return;
			}
			else {
				// replace image uploaders with file ones
				foreach ( $old_fields as $field ) {
					if ( isset( $field['input_type'] ) && $field['input_type'] == 'image_upload' ) {
						$field['input_type'] = 'file_upload';
					}
					if ( isset( $field['template'] ) && $field['template'] == 'image_upload' ) {
						$field['template'] = 'file_upload';
					}
				}
				update_post_meta( $this->new_settings['fes-submission-form'], 'fes-form', $old_fields );
			}
		}


		// if profile form
		if ( isset( $this->new_settings['fes-profile-form'] ) && $this->new_settings['fes-profile-form'] != '' ) {
			// add fields to profile form
			$old_fields = get_post_meta( $this->new_settings['fes-profile-form'], 'fes-form', true );
			$nextindex = 1;
			if ( !is_array( $old_fields ) ) {
				$old_fields = array();
			} else {
				// replace image uploaders with file ones
				foreach ( $old_fields as $field ) {
					if ( isset( $field['input_type'] ) && $field['input_type'] == 'image_upload' ) {
						$field['input_type'] = 'file_upload';
					}
					if ( isset( $field['template'] ) && $field['template'] == 'image_upload' ) {
						$field['template'] = 'file_upload';
					}
				}

				end( $old_fields );
				$last = key( $old_fields );
				$nextindex = $last + 1;
			}

			$old_fields[$nextindex] = array(
				'template' => 'text',
				'required' => 'yes',
				'label' => 'Name of Store',
				'name' => 'name_of_store',
				'is_meta' => 'yes',
				'help' => 'What would you like your store to be called?',
				'css' => '',
				'placeholder' => '',
				'default' => '',
				'size' => '40'
			);
			$nextindex++;
			$old_fields[$nextindex] = array(
				'template' => 'email',
				'required' => 'yes',
				'label' => 'Email to use for Contact Form',
				'name' => 'email_to_use_for_contact_form',
				'is_meta' => 'yes',
				'help' => 'This email, if filled in will be used for the vendor contact forms. if it is not filled in, the one from your user profile will be used.',
				'css' => '',
				'placeholder' => '',
				'default' => '',
				'size' => '40'
			);
			update_post_meta( $this->new_settings['fes-profile-form'], 'fes-form', $old_fields );
		}


		// foreach fes_form if has editor in the name, remove it ( only affects FES Forms from pre 2.2 )
		// Submission form
		if ( isset( $this->new_settings['fes-submission-form'] ) && $this->new_settings['fes-submission-form'] != '' ) {
			$id = $this->new_settings['fes-submission-form'];
			$update = array(
				'ID'           => $id,
				'post_title'   => __( 'Submission Form', 'edd_fes' ),
				'post_name'    =>'fes-submission-form'
			);
			wp_update_post( $update );
		}

		// Profile form
		if ( isset( $this->new_settings['fes-profile-form'] ) && $this->new_settings['fes-profile-form'] != '' ) {
			$id = $this->new_settings['fes-profile-form'];
			$update = array(
				'ID'           => $id,
				'post_title'   => __( 'Profile Form', 'edd_fes' ),
				'post_name'    =>'fes-profile-form'
			);
			wp_update_post( $update );
		}

	}

	public function fes_v23_upgrades() {
		// Create Vendor Database table
		$db = new FES_DB_Vendors();
		$db->create_table();

		// Foreach form, ensure the field has a template and remove input_type. Also ensure password fields have names
		// submission form
		if ( isset( $this->new_settings['fes-submission-form'] ) && $this->new_settings['fes-submission-form'] != '' ) {
			$old_fields = get_post_meta( $this->new_settings['fes-submission-form'], 'fes-form', true );
			if ( is_array( $old_fields ) ) {
				foreach ( $old_fields as $id => $field ) {
					$field = fes_upgrade_field( $field ); // upgrade field
					$old_fields[ $id ] = $field; // save new field back
				}
				update_post_meta( $this->new_settings['fes-submission-form'], 'fes-form', $old_fields );
				fes_save_initial_submission_form( $this->new_settings['fes-submission-form'], false );
			}
		} else {
			$this->create_submission_form();
		}

		// profile form
		if ( isset( $this->new_settings['fes-profile-form'] ) && $this->new_settings['fes-profile-form'] != '' ) {
			$old_fields = get_post_meta( $this->new_settings['fes-profile-form'], 'fes-form', true );
			if ( is_array( $old_fields ) ) {
				foreach ( $old_fields as $id => $field ) {
					$field = fes_upgrade_field( $field ); // upgrade field
					$old_fields[ $id ] = $field; // save new field back
				}
				update_post_meta( $this->new_settings['fes-profile-form'], 'fes-form', $old_fields );
				fes_save_initial_profile_form( $this->new_settings['fes-profile-form'], false );
			}
		} else {
			$this->create_profile_form();
		}

		// registration form
		if ( isset( $this->new_settings['fes-registration-form'] ) && $this->new_settings['fes-registration-form'] != '' ) {
			$old_fields = get_post_meta( $this->new_settings['fes-registration-form'], 'fes-form', true );
			if ( is_array( $old_fields ) ) {
				foreach ( $old_fields as $id => $field ) {
					$field = fes_upgrade_field( $field ); // upgrade field
					$old_fields[ $id ] = $field; // save new field back
				}
				update_post_meta( $this->new_settings['fes-registration-form'], 'fes-form', $old_fields );
				fes_save_initial_registration_form( $this->new_settings['fes-registration-form'], false );
			}
		} else {
			$this->create_registration_form();
		}

		// login form
		if ( isset( $this->new_settings['fes-login-form'] ) && $this->new_settings['fes-login-form'] != '' ) {
			fes_save_initial_login_form( $this->new_settings['fes-login-form'] );
		} else {
			$this->create_login_form();
		}

		//  vendor contact form
		if ( isset( $this->new_settings['fes-vendor-contact-form'] ) && $this->new_settings['fes-vendor-contact-form'] != '' ) {
			fes_save_initial_vendor_contact_form( $this->new_settings['fes-vendor-contact-form'] );
		} else {
			$this->create_vendor_contact_form();
		}

		// delete all fes-forms not in the 5 above
		$forms = get_posts( array( 'post_type' => 'fes-forms', 'fields' => 'ids', 'posts_per_page' => -1, '' ) );
		if ( $forms ) {
			foreach ( $forms as $form ) {
				if ( isset( $this->new_settings['fes-submission-form'] )          && $this->new_settings['fes-submission-form'] == $form  && $this->new_settings['fes-submission-form'] ) {
					continue;
				}
				else if ( isset( $this->new_settings['fes-profile-form'] )        && $this->new_settings['fes-profile-form'] == $form       && $this->new_settings['fes-profile-form'] ) {
						continue;
					}
				else if ( isset( $this->new_settings['fes-registration-form'] )   && $this->new_settings['fes-registration-form'] == $form   && $this->new_settings['fes-registration-form'] ) {
						continue;
					}
				else if ( isset( $this->new_settings['fes-login-form'] )     && $this->new_settings['fes-login-form'] == $form    && $this->new_settings['fes-login-form'] ) {
						continue;
					}
				else if ( isset( $this->new_settings['fes-vendor-contact-form'] ) && $this->new_settings['fes-vendor-contact-form'] == $form && $this->new_settings['fes-vendor-contact-form'] ) {
						continue;
					}
				else {
					wp_delete_post( $form, true );
				}
			}
		}

		$old_settings = get_option( 'fes_settings', array() );

		// Set the new vendor constant field
		if ( isset( $old_settings['fes-plugin-constants'] ) && isset( $old_settings['fes-plugin-constants'][4] ) && $old_settings['fes-plugin-constants'][4] != '' ) {
			$this->new_settings['fes-vendor-constant'] = $old_settings['fes-plugin-constants'][4];
		}

		// Set the new product constant field
		if ( isset( $old_settings['fes-plugin-constants'] ) && isset( $old_settings['fes-plugin-constants'][8] ) && $old_settings['fes-plugin-constants'][8] != '' ) {
			$this->new_settings['fes-product-constant'] = $old_settings['fes-plugin-constants'][8];
		}

		// delete old settings option from 2.1
		delete_option( 'edd_fes_options' );

		// delete all old applications
		$posts = get_posts( array(
				'nopaging'    => true,
				'orderby'     => 'title',
				'post_type'   => 'fes-applications',
				'post_status' => 'any',
				'order'    => 'ASC'
			) );
		if ( $posts ) {
			foreach ( $posts as $post ) {
				wp_delete_post( $post->ID, true );
			}
		}
	}

	public function create_vendor_dashboard_page() {
		global $wpdb, $wp_version;
		$page_id = isset( $this->new_settings['fes-vendor-dashboard-page'] ) ? $this->new_settings['fes-vendor-dashboard-page'] : 0;
		if ( $page_id > 0 && get_post( $page_id ) ) {
			return;
		}
		$page_found = $wpdb->get_var( $wpdb->prepare( "SELECT ID FROM " . $wpdb->posts . " WHERE post_name = %s LIMIT 1;", "vendor-dashboard" ) );
		if ( $page_found ) {
			if ( !$page_id ) {
				$this->new_settings['fes-vendor-dashboard-page'] = $page_found;
				return;
			}
			return;
		}
		$page_data = array(
			'post_status' => 'publish',
			'post_type' => 'page',
			'post_author' => get_current_user_id(),
			'post_name' => 'vendor-dashboard',
			'post_title' => __( 'Vendor Dashboard', 'edd_fes' ),
			'post_content' => '[fes_vendor_dashboard]',
			'comment_status' => 'closed'
		);
		$page_id   = wp_insert_post( $page_data );
		$this->new_settings['fes-vendor-dashboard-page'] = $page_id;
	}

	public function create_vendor_page() {
		global $wpdb, $wp_version;
		$page_id = isset( $this->new_settings['fes-vendor-page'] ) ? $this->new_settings['fes-vendor-page'] : 0;
		if ( $page_id > 0 && get_post( $page_id ) ) {
			return;
		}
		$page_found = $wpdb->get_var( $wpdb->prepare( "SELECT ID FROM " . $wpdb->posts . " WHERE post_name = %s LIMIT 1;", "vendor" ) );
		if ( $page_found ) {
			if ( !$page_id ) {
				$this->new_settings['fes-vendor-page'] = $page_found;
				return;
			}
			return;
		}
		$page_data = array(
			'post_status'    => 'publish',
			'post_type'      => 'page',
			'post_author'    => get_current_user_id(),
			'post_name'   => 'vendor',
			'post_title'   => __( 'Vendor', 'edd_fes' ),
			'post_content'   => '[downloads]',
			'comment_status' => 'closed'
		);
		$page_id   = wp_insert_post( $page_data );
		$this->new_settings['fes-vendor-page'] = $page_id;
	}

	public function create_submission_form( ) {
		global $wpdb, $wp_version;
		$slug = "fes-submission-form";
		$page_id = isset( $this->new_settings[$slug] ) ? $this->new_settings[$slug] : 0;
		if ( $page_id > 0 && get_post( $page_id ) ) {
			return;
		}
		$page_found = $wpdb->get_var( $wpdb->prepare( "SELECT ID FROM " . $wpdb->posts . " WHERE post_name = %s LIMIT 1;", $slug ) );
		if ( $page_found ) {
			if ( !$page_id ) {
				$this->new_settings['fes-submission-form'] = $page_found;
				return;
			}
			return;
		}

		$page_data = array(
			'post_status' => 'publish',
			'post_type' => 'fes-forms',
			'post_author' => get_current_user_id(),
			'post_title' => __( 'Submission Form', 'edd_fes' )
		);
		$page_id   = wp_insert_post( $page_data );
		fes_save_initial_submission_form( $page_id );
		$this->new_settings['fes-submission-form'] = $page_id;
	}
	public function create_profile_form() {
		global $wpdb, $wp_version;
		$slug = "fes-profile-form";
		$page_id = isset( $this->new_settings[$slug] ) ? $this->new_settings[$slug] : 0;
		if ( $page_id > 0 && get_post( $page_id ) ) {
			return;
		}
		$page_found = $wpdb->get_var( $wpdb->prepare( "SELECT ID FROM " . $wpdb->posts . " WHERE post_name = %s LIMIT 1;", $slug ) );
		if ( $page_found ) {
			if ( !$page_id ) {
				$this->new_settings[$slug] = $page_found;
				return;
			}
			return;
		}

		$page_data = array(
			'post_status' => 'publish',
			'post_type' => 'fes-forms',
			'post_author' => get_current_user_id(),
			'post_title' => __( 'Profile Form', 'edd_fes' )
		);
		$page_id   = wp_insert_post( $page_data );
		fes_save_initial_profile_form( $page_id );
		$this->new_settings['fes-profile-form'] = $page_id;
	}

	public function create_registration_form() {
		global $wpdb, $wp_version;
		$slug = "fes-registration-form";
		$page_id = isset( $this->new_settings[$slug] ) ? $this->new_settings[$slug] : 0;
		if ( $page_id > 0 && get_post( $page_id ) ) {
			return;
		}
		$page_found = $wpdb->get_var( $wpdb->prepare( "SELECT ID FROM " . $wpdb->posts . " WHERE post_name = %s LIMIT 1;", $slug ) );
		if ( $page_found ) {
			if ( !$page_id ) {
				$this->new_settings[$slug] = $page_found;
				return;
			}
			return;
		}

		$page_data = array(
			'post_status' => 'publish',
			'post_type' => 'fes-forms',
			'post_author' => get_current_user_id(),
			'post_title' => __( 'Registration Form', 'edd_fes' )
		);
		$page_id   = wp_insert_post( $page_data );
		fes_save_initial_registration_form( $page_id );
		$this->new_settings['fes-registration-form'] = $page_id;
	}

	public function create_login_form() {
		global $wpdb, $wp_version;
		$slug = "fes-login-form";
		$page_id = isset( $this->new_settings[$slug] ) ? $this->new_settings[$slug] : 0;
		if ( $page_id > 0 && get_post( $page_id ) ) {
			return;
		}
		$page_found = $wpdb->get_var( $wpdb->prepare( "SELECT ID FROM " . $wpdb->posts . " WHERE post_name = %s LIMIT 1;", $slug ) );
		if ( $page_found ) {
			if ( !$page_id ) {
				$this->new_settings['fes-login-form'] = $page_found;
				return;
			}
			return;
		}

		$page_data = array(
			'post_status' => 'publish',
			'post_type' => 'fes-forms',
			'post_author' => get_current_user_id(),
			'post_title' => __( 'Login Form', 'edd_fes' )
		);
		$page_id   = wp_insert_post( $page_data );
		fes_save_initial_login_form( $page_id );
		$this->new_settings['fes-login-form'] = $page_id;
	}

	public function create_vendor_contact_form() {
		global $wpdb, $wp_version;
		$slug = "fes-vendor-contact-form";
		$page_id = isset( $this->new_settings[$slug] ) ? $this->new_settings[$slug] : 0;
		if ( $page_id > 0 && get_post( $page_id ) ) {
			return;
		}
		$page_found = $wpdb->get_var( $wpdb->prepare( "SELECT ID FROM " . $wpdb->posts . " WHERE post_name = %s LIMIT 1;", $slug ) );
		if ( $page_found ) {
			if ( !$page_id ) {
				$this->new_settings['fes-vendor-contact-form'] = $page_found;
				return;
			}
			return;
		}

		$page_data = array(
			'post_status' => 'publish',
			'post_type' => 'fes-forms',
			'post_author' => get_current_user_id(),
			'post_title' => __( 'Vendor Contact Form', 'edd_fes' )
		);
		$page_id   = wp_insert_post( $page_data );
		fes_save_initial_vendor_contact_form( $page_id );
		$this->new_settings['fes-vendor-contact-form'] = $page_id;
	}

	public function add_new_roles() {
		global $wp_roles;
		remove_role( 'frontend_vendor' );
		add_role( 'frontend_vendor', 'Frontend Vendor', array(
			'read' => true,
			'edit_posts' => true,
			'upload_files' => true,
		) );
		if ( class_exists( 'WP_Roles' ) && !isset( $wp_roles ) )
			$wp_roles = new WP_Roles();
		if ( is_object( $wp_roles ) ) {
			$capabilities     = array();
			$capability_types = array(
				'product'
			);
			foreach ( $capability_types as $capability_type ) {
				$capabilities[ $capability_type ] = array(
					// Post type
					"edit_{$capability_type}",
					"read_{$capability_type}",
					"delete_{$capability_type}",
					"edit_{$capability_type}s",
					"read_private_{$capability_type}s",
					"edit_private_{$capability_type}s",
					// Terms
					"manage_{$capability_type}_terms",
					"edit_{$capability_type}_terms",
					"assign_{$capability_type}_terms"
				);
			}
			foreach ( $capabilities as $cap_group ) {
				foreach ( $cap_group as $cap ) {
					$wp_roles->add_cap( 'frontend_vendor', $cap );
				}
			}
			$wp_roles->add_cap( 'frontend_vendor', 'edit_product' );
			$wp_roles->add_cap( 'frontend_vendor', 'edit_products' );
			$wp_roles->add_cap( 'frontend_vendor', 'upload_files' );
			$wp_roles->add_cap( 'frontend_vendor', 'assign_product_terms' );
			$wp_roles->add_cap( 'frontend_vendor', 'delete_product' );
			$wp_roles->add_cap( 'frontend_vendor', 'delete_products' );

			$wp_roles->add_cap( 'administrator', 'fes_is_admin' );
			$wp_roles->add_cap( 'editor', 'fes_is_admin' );
		}
	}
}
