<?php
if ( !class_exists( "FES_Settings" ) ) {

    class FES_Settings {

        public $args = array();
        public $sections = array();
        public $ReduxFramework;
        public function __construct() {

            if ( !class_exists( "ReduxFramework" ) ) {
                require_once( dirname( __FILE__ ) . '/redux/ReduxCore/framework.php' );
            }

            $this->initSettings();
        }

        public function initSettings() {
            // Set the default arguments
            $this->setArguments();

            // Create the sections and fields
            $this->setSections();

            if ( !isset( $this->args['opt_name'] ) ) { // No errors please
                return;
            }

            $this->ReduxFramework = new ReduxFramework( $this->sections, $this->args );
        }


        public function setSections() {
            $this->sections[] = array(
                'title' => __( 'Main Settings', 'edd_fes' ),
                'desc' => __( 'Easy Digital Downloads Frontend Submissions is a constantly improving piece of complex software. For support and feature requests contact us <a href="https://easydigitaldownloads.com/support/">using this form</a>. For FES\'s official roadmap visit <a href="https://trello.com/b/vT8aivfj/fes-development">our Trello board</a>.', 'edd_fes' ),
                'icon' => 'el-icon-home',
                'fields' => array(
                    array(
                        'id' => 'fes-vendor-constant',
                        'type' => 'text',
                        'title' => __( 'Rename "vendor"', 'edd_fes' ),
                        'subtitle' => __( 'This will change all mentions of "vendor" within FES', 'edd_fes' ),
                        'validate' => 'no_html',
                        'default' => 'vendor',
                    ),
                    array(
                        'id' => 'fes-product-constant',
                        'type' => 'text',
                        'title' => __( 'Rename "product"', 'edd_fes' ),
                        'subtitle' => __( 'This will change all mentions of "product" within FES', 'edd_fes' ),
                        'validate' => 'no_html',
                        'default' => 'product',
                    ),
                    array(
                        'id'=>'fes-dashboard-notification',
                        'type' => 'textarea',
                        'title' => __( 'Vendor Announcement', 'edd_fes' ),
                        'subtitle' => __( 'Use this to announce things to your vendors. Appears on the Vendor Dashboard Page once logged in.', 'edd_fes' ),
                        'validate' => 'html',
                        'default' => __( 'This is the vendor dashboard. Add welcome text or any other information that is applicable to your vendors.', 'edd_fes' )
                    ),
                    array(
                        'id'=> 'fes-use-css',
                        'type' => 'switch',
                        'title' => __( 'Use FES\'s CSS', 'edd_fes' ),
                        'subtitle' => __( 'You can use this to turn off FES\'s CSS if you want. We highly don\'t recommend doing that', 'edd_fes' ),
                        'default'   => true,
                    ),
                )
            );

            $this->sections[] = array(
                'icon' => 'el-icon-list-alt',
                'title' => __( 'Forms/Pages', 'edd_fes' ),
                'desc' => __( '<p class="description">Settings for Pages and Forms</p>', 'edd_fes' ),
                'fields' => array(
                    array(
                        'id' => 'fes-vendor-dashboard-page',
                        'type' => 'select',
                        'data' => 'pages',
                        'title' => __( 'Vendor Dashboard Page', 'edd_fes' ),
                        'subtitle'=> __( 'This setting is used by FES to determine which page is the vendor dashboard. Only change if you are using a custom dashboard page.', 'edd_fes' ),
                        'desc' => __( 'Contains the [fes_vendor_dashboard] shortcode', 'edd_fes' ),
                    ),
                    array(
                        'id' => 'fes-vendor-page',
                        'type' => 'select',
                        'data' => 'pages',
                        'title' => __( 'Vendor Page', 'edd_fes' ),
                        'subtitle'=> __( 'This setting is used by FES to determine which page is the vendor store page. Only change if you are using a custom vendor store page.', 'edd_fes' ),
                        'desc' => __( 'Contains the [downloads] shortcode', 'edd_fes' ),
                    ),
                    array(
                        'id'=> 'fes-allow-multiple-purchase-mode',
                        'type' => 'switch',
                        'title' => __( 'Enable Multiple Purchase Mode for all vendor products', 'edd_fes' ),
                        'subtitle' => __( 'If on, allows customers to purchase multiple variations of a vendor product at a time', 'edd_fes' ),
                        'default'       => 0,
                    ),
                    array(
                        'id'=> 'fes-show-custom-meta',
                        'type' => 'switch',
                        'title' => __( 'Show custom fields on the download?', 'edd_fes' ),
                        'subtitle' => __( 'You can select which fields to show on the download page using the radio buttons on the top of each field on the submission formbuilder.', 'edd_fes' ),
                        'default'   => false,
                    ),
                    array(
                        'id'=> 'fes-show-forms',
                        'type' => 'switch',
                        'title' => __( 'Display settings panel form assignments', 'edd_fes' ),
                        'subtitle'=> __( 'Toggles on the behind the scenes settings for form page id\'s. Only needed for FES support purposes.', 'edd_fes' ),
                        'default'       => 0,
                    ),
                    array(
                        'id' => 'fes-submission-form',
                        'type'     => 'select',
                        'data' => 'posts',
                        'args'=> array( 'post_type'=> 'fes-forms', 'posts_per_page'=> -1),
                        'title' => __( 'Submission Form', 'edd_fes' ),
                        'required' => array( 'fes-show-forms', 'equals', array( '1' ) ),
                    ),
                    array(
                        'id' => 'fes-profile-form',
                        'type'     => 'select',
                        'data' => 'posts',
                        'args'=> array( 'post_type'=> 'fes-forms', 'posts_per_page'=>-1),
                        'title' => __( 'Profile Form', 'edd_fes' ),
                        'required' => array( 'fes-show-forms', 'equals', array( '1' ) ),
                    ),
                    array(
                        'id' => 'fes-login-form',
                        'type'     => 'select',
                        'data' => 'posts',
                        'args'=> array( 'post_type'=> 'fes-forms', 'posts_per_page'=>-1),
                        'title' => __( 'Login Form', 'edd_fes' ),
                        'required' => array( 'fes-show-forms', 'equals', array( '1' ) ),
                    ),
                    array(
                        'id' => 'fes-registration-form',
                        'type'     => 'select',
                        'data' => 'posts',
                        'args'=> array( 'post_type'=> 'fes-forms', 'posts_per_page'=>-1),
                        'title' => __( 'Registration Form', 'edd_fes' ),
                        'required' => array( 'fes-show-forms', 'equals', array( '1' ) ),
                    ),
                    array(
                        'id' => 'fes-vendor-contact-form',
                        'type'     => 'select',
                        'data' => 'posts',
                        'args'=> array( 'post_type'=> 'fes-forms', 'posts_per_page'=>-1),
                        'title' => __( 'Vendor Contact Form', 'edd_fes' ),
                        'required' => array( 'fes-show-forms', 'equals', array( '1' ) ),
                    ),                    
                )
            );

            $this->sections[] = array(
                'icon' => 'el-icon-check',
                'title' => __( 'Permissions', 'edd_fes' ),
                'desc' => __( '<p class="description">Settings for FES Permissions</p>', 'edd_fes' ),
                'fields' => array(
                    array(
                        'id'=> 'fes-allow-registrations',
                        'type' => 'switch',
                        'title' => __( 'Registration', 'edd_fes' ),
                        'subtitle'=> __( 'Allow guests to apply to become a vendor', 'edd_fes' ),
                        'default'       => 1,
                    ),
                    array(
                        'id'=> 'fes-allow-applications',
                        'type' => 'switch',
                        'title' => __( 'Applications', 'edd_fes' ),
                        'subtitle'=> __( 'Allow existing WordPress users to apply to become a vendor', 'edd_fes' ),
                        'default'       => 1,
                    ),
                    array(
                        'id'=> 'fes-allow-backend-access',
                        'type' => 'switch',
                        'title' => __( 'Allow WP Backend Access to Vendors?', 'edd_fes' ),
                        'subtitle'=> __( 'Does not remove the admin bar. Simply prevents vendors from accessing the backend.', 'edd_fes' ),
                        'default'       => 0,
                    ),
                    array(
                        'id'=> 'fes-remove-admin-bar',
                        'type' => 'switch',
                        'title' => __( 'Allow Vendors to See the Admin Bar?', 'edd_fes' ),
                        'subtitle'=> __( 'This only removes the bar for users with the frontend_vendor cap but not the fes_is_admin cap', 'edd_fes' ),
                        'default'       => 0,
                    ),
                    array(
                        'id'=> 'fes-auto-approve-vendors',
                        'type' => 'switch',
                        'title' => __( 'Automatically Approve Vendors?', 'edd_fes' ),
                        'subtitle'=> __( 'If off, vendors will not be able to submit products until you manually approve them', 'edd_fes' ),
                        'default'       => 1,
                    ),
                    array(
                        'id'=> 'fes-auto-approve-submissions',
                        'type' => 'switch',
                        'title' => __( 'Automatically Approve Submissions?', 'edd_fes' ),
                        'subtitle'=> __( 'If off, vendor products will be saved as a pending download until you manually approve them', 'edd_fes' ),
                        'default'       => 0,
                    ),
                    array(
                        'id'=> 'fes-auto-approve-edits',
                        'type' => 'switch',
                        'title' => __( 'Automatically Approve Vendor Edits?', 'edd_fes' ),
                        'subtitle'=> __( 'If off, vendor products will be changed to pending status (removed from store) until manually approved', 'edd_fes' ),
                        'default'       => 1,
                    ),
                    array(
                        'id'=> 'fes-allow-vendors-to-create-products',
                        'type' => 'switch',
                        'title' => __( 'Allow Vendors to Create Products?', 'edd_fes' ),
                        'subtitle'=> __( 'If on, vendors will be able to create products', 'edd_fes' ),
                        'default'       => 1,
                    ),                    
                    array(
                        'id'=> 'fes-allow-vendors-to-edit-products',
                        'type' => 'switch',
                        'title' => __( 'Allow Vendors to Edit Products?', 'edd_fes' ),
                        'subtitle'=> __( 'If on, vendors will be able to edit their products. If vendors cannot edit products, after submitting a new product they will be redirected to the new product form, instead of being default redirected to edit the product they just submitted.', 'edd_fes' ),
                        'default'       => 1,
                    ),
                    array(
                        'id'=> 'fes-allow-vendors-to-delete-products',
                        'type' => 'switch',
                        'title' => __( 'Allow Vendors to Delete Products?', 'edd_fes' ),
                        'subtitle'=> __( 'If on, vendors will be able to delete their products', 'edd_fes' ),
                        'default'       => 1,
                    ),
                    array(
                        'id'=> 'fes-allow-vendors-to-view-orders',
                        'type' => 'switch',
                        'title' => __( 'Allow Vendors to View Orders?', 'edd_fes' ),
                        'subtitle'=> __( 'If on, vendors will be able to view orders', 'edd_fes' ),
                        'default'       => 1,
                    ),
                )
            );
            $this->sections[] = array(
                'icon' => 'el-icon-website',
                'title' => __( 'Emails', 'edd_fes' ),
                'desc' => __( '<p class="description">Settings for FES Emails</p>', 'edd_fes' ),
                'fields' => array(
                    array(
                        'id'   =>'fes-admin-vendor-email-divider',
                        'title' => __( 'Emails to Admin', 'edd_fes' ),
                        'type' => 'section',
                        'indent'   => true,
                    ),
                            array(
                                'id'=> 'fes-admin-new-app-email-toggle',
                                'type' => 'switch',
                                'title' => __( 'Vendor Application', 'edd_fes' ),
                                'subtitle' => __( 'If the "auto approve vendors" setting is off, this email is sent when a person applies to become a vendor. If the "auto approve vendors" setting is on, this email is sent when a new vendor joins the site.' , 'edd_fes' ),
                                'default'       => 1,
                            ),
                            array(
                                'id' => 'fes-admin-new-app-email',
                                'type' => 'textarea',
                                'title' => __( 'Vendor Application Email Contents', 'edd_fes' ),
                                'subtitle' => __( 'This setting sets what is in the email body for the vendor application email.', 'edd_fes' ),
                                'desc' => __( 'A list of shortcodes you can use is ', 'edd_fes' ).'<a href="http://docs.easydigitaldownloads.com/article/338-frontend-submissions-email-configuration" target="_blank" >'.__(' in our documentation','edd_fes').'</a>',
                                'validate' => 'html',
                                'required' =>  array( 'fes-admin-new-app-email-toggle', 'equals',  array( '1' ) ),
                                'default' => '{fullname} has applied to become a vendor.',
                            ),
                            array(
                                'id'=> 'fes-admin-new-submission-email-toggle',
                                'type' => 'switch',
                                'title' => __( 'New Product', 'edd_fes' ),
                                'subtitle' => __( 'This email is sent when a vendor submits a new product to be reviewed (and the auto approve submissions setting is off)', 'edd_fes' ),
                                'default'       => 1,
                            ),
                            array(
                                'id' => 'fes-admin-new-submission-email',
                                'type' => 'textarea',
                                'title' => __( 'New Product Email Contents', 'edd_fes' ),
                                'subtitle' => __( 'This setting sets what is in the email body for the new product email.', 'edd_fes' ),
                                'desc' => __( 'A list of shortcodes you can use is ', 'edd_fes' ).'<a href="http://docs.easydigitaldownloads.com/article/338-frontend-submissions-email-configuration" target="_blank" >'.__(' in our documentation','edd_fes').'</a>',
                                'validate' => 'html',
                                'required' =>  array( 'fes-admin-new-submission-email-toggle', 'equals', array( '1' ) ),
                                'default' => 'Vendor {fullname} has submitted the following submission for review: 
        Name of Submission: {post-title} 
        Content of Submission: {post-content} 
        Submission Category: {post-categories} n 
        Submission Tags: {post-tags} 
        Date of Submission: {post-date}',
                            ),
                            array(
                                'id'=> 'fes-admin-new-submission-edit-email-toggle',
                                'type' => 'switch',
                                'title' => __( 'Edited Product', 'edd_fes' ),
                                'subtitle' => __( 'This email is sent when a vendor submits an edit to an existing product to be reviewed (and the auto approve edits setting is off)', 'edd_fes' ),
                                'default'       => 1,
                                'required' =>  array( 'fes-auto-approve-edits', 'equals', array( '0' ) ),
                            ),
                            array(
                                'id' => 'fes-admin-new-submission-edit-email',
                                'type' => 'textarea',
                                'title' => __( 'Edited Product Email Contents', 'edd_fes' ),
                                'subtitle' => __( 'This setting sets what is in the email body for the edited product email.', 'edd_fes' ),
                                'desc' => __( 'A list of shortcodes you can use is ', 'edd_fes' ).'<a href="http://docs.easydigitaldownloads.com/article/338-frontend-submissions-email-configuration" target="_blank" >'.__(' in our documentation','edd_fes').'</a>',
                                'validate' => 'html',
                                'required' =>  array( 'fes-admin-new-submission-edit-email-toggle', 'equals', array( '1' ) ),
                                'default' => 'Vendor {fullname} has submitted the following edit for review: 
        Name of Submission: {post-title} 
        Content of Submission: {post-content} 
        Submission Category: {post-categories} n 
        Submission Tags: {post-tags}',
                            ),
                    array(
                        'id'   =>'fes-vendor-registration-email-divider',
                        'title' => __( 'Emails to Vendors', 'edd_fes' ),
                        'type' => 'section',
                        'indent'   => true,
                    ),
                            array(
                                'id'=> 'fes-vendor-new-app-email-toggle',
                                'type' => 'switch',
                                'title' => __( 'Application Received', 'edd_fes' ),
                                'subtitle' => __( 'This email is sent when a person applies to become a vendor and the "auto approve vendors" setting is off.', 'edd_fes' ),
                                'default'       => 1,
                                'required' =>  array( 'fes-auto-approve-submissions', 'equals', array( '0' ) ),
                            ),
                            array(
                                'id' => 'fes-vendor-new-app-email',
                                'type' => 'textarea',
                                'title' => __( 'Application Received Email Contents', 'edd_fes' ),
                                'subtitle' => __( 'This setting sets what is in the email body for the application received email.', 'edd_fes' ),
                                'desc' => __( 'A list of shortcodes you can use is ', 'edd_fes' ).'<a href="http://docs.easydigitaldownloads.com/article/338-frontend-submissions-email-configuration" target="_blank" >'.__(' in our documentation','edd_fes').'</a>',
                                'validate' => 'html',
                                'required' =>  array( 'fes-vendor-new-app-email-toggle', 'equals', array( '1' ) ),
                                'default' => 'Dear {firstname}, 
        Your application to become a vendor has been received.

        We will process it as soon as possible.',
                            ),
                            array(
                                'id'=> 'fes-vendor-app-approved-email-toggle',
                                'type' => 'switch',
                                'title' => __( 'Application Approved', 'edd_fes' ),
                                'subtitle' => __( 'This email is sent when a person\'s application to become a vendor is approved and the "auto approve vendors" setting is off.', 'edd_fes' ),
                                'default'       => 1,
                                'required' =>  array( 'fes-auto-approve-vendors', 'equals', array( '0' ) ),
                            ),
                            array(
                                'id' => 'fes-vendor-app-approved-email',
                                'type' => 'textarea',
                                'title' => __( 'Application Approved Email Contents', 'edd_fes' ),
                                'subtitle' => __( 'This setting sets what is in the email body for the application approved email.', 'edd_fes' ),
                                'desc' => __( 'A list of shortcodes you can use is ', 'edd_fes' ).'<a href="http://docs.easydigitaldownloads.com/article/338-frontend-submissions-email-configuration" target="_blank" >'.__(' in our documentation','edd_fes').'</a>',
                                'validate' => 'html',
                                'required' =>  array( 'fes-vendor-app-approved-email-toggle', 'equals', array( '1' ) ),
                                'default' => 'Dear {firstname}, 
        Your application to become a vendor has been approved.',
                            ),
                            array(
                                'id'=> 'fes-vendor-app-declined-email-toggle',
                                'type' => 'switch',
                                'title' => __( 'Application Declined', 'edd_fes' ),
                                'subtitle' => __( 'This email is sent when a person\'s application to become a vendor is declined and the "auto approve vendors" setting is off', 'edd_fes' ),
                                'default'       => 1,
                                'required' =>  array( 'fes-auto-approve-vendors', 'equals', array( '0' ) ),
                            ),
                            array(
                                'id' => 'fes-vendor-app-declined-email',
                                'type' => 'textarea',
                                'title' => __( 'Application Declined Email Contents', 'edd_fes' ),
                                'subtitle' => __( 'This setting sets what is in the email body for the application declined email.', 'edd_fes' ),
                                'desc' => __( 'A list of shortcodes you can use is ', 'edd_fes' ).'<a href="http://docs.easydigitaldownloads.com/article/338-frontend-submissions-email-configuration" target="_blank" >'.__(' in our documentation','edd_fes').'</a>',
                                'validate' => 'html',
                                'required' =>  array( 'fes-vendor-app-declined-email-toggle', 'equals', array( '1' ) ),
                                'default' => 'Dear {firstname}, 
        Your application to become a vendor has been declined!',
                            ),
                            array(
                                'id'=> 'fes-vendor-new-auto-vendor-email-toggle',
                                'type' => 'switch',
                                'title' => __( 'Application Auto-Approved', 'edd_fes' ),
                                'subtitle' => __( 'This email is sent when a person becomes a vendor because the "auto approve vendors" setting is on', 'edd_fes' ),
                                'title' => __( 'Send Vendor Email Upon Being Auto-Approved?', 'edd_fes' ),
                                'default'       => 1,
                                'required' =>  array( 'fes-auto-approve-vendors', 'equals', array( '1' ) ),
                            ),
                            array(
                                'id' => 'fes-vendor-new-auto-vendor-email',
                                'type' => 'textarea',
                                'title' => __( 'Application Auto-Approved Email Contents', 'edd_fes' ),
                                'subtitle' => __( 'This setting sets what is in the email body for the application auto-approved email.', 'edd_fes' ),
                                'desc' => __( 'A list of shortcodes you can use is ', 'edd_fes' ).'<a href="http://docs.easydigitaldownloads.com/article/338-frontend-submissions-email-configuration" target="_blank" >'.__(' in our documentation','edd_fes').'</a>',
                                'validate' => 'html',
                                'required' =>  array( 'fes-vendor-new-auto-vendor-email-toggle', 'equals', array( '1' ) ),
                                'default' => 'Dear {firstname}, 
        Your application to become a vendor has been approved.',
                            ),
                            array(
                                'id'=> 'fes-vendor-app-revoked-email-toggle',
                                'type' => 'switch',
                                'title' => __( 'Vendor Revoked', 'edd_fes' ),
                                'subtitle' => __( 'This email is sent when vendor is revoked', 'edd_fes' ),
                                'default'       => 1,
                            ),
                            array(
                                'id' => 'fes-vendor-app-revoked-email',
                                'type' => 'textarea',
                                'title' => __( 'Vendor Revoked Email Contents', 'edd_fes' ),
                                'subtitle' => __( 'This setting sets what is in the email body for the vendor revoked email.', 'edd_fes' ),
                                'desc' => __( 'A list of shortcodes you can use is ', 'edd_fes' ).'<a href="http://docs.easydigitaldownloads.com/article/338-frontend-submissions-email-configuration" target="_blank" >'.__(' in our documentation','edd_fes').'</a>',
                                'validate' => 'html',
                                'required' =>  array( 'fes-vendor-app-revoked-email-toggle', 'equals', array( '1' ) ),
                                'default' => 'Dear {firstname}, 
        Your vendor account has been revoked.',
                            ),
                            array(
                                'id'=> 'fes-vendor-suspended-email-toggle',
                                'type' => 'switch',
                                'title' => __( 'Vendor Suspended', 'edd_fes' ),
                                'subtitle' => __( 'This email is sent when vendor is suspended', 'edd_fes' ),
                                'default'       => 1,
                            ),
                            array(
                                'id' => 'fes-vendor-suspended-email',
                                'type' => 'textarea',
                                'title' => __( 'Vendor Suspended Email Contents', 'edd_fes' ),
                                'subtitle' => __( 'This setting sets what is in the email body for the vendor suspended email.', 'edd_fes' ),
                                'desc' => __( 'A list of shortcodes you can use is ', 'edd_fes' ).'<a href="http://docs.easydigitaldownloads.com/article/338-frontend-submissions-email-configuration" target="_blank" >'.__(' in our documentation','edd_fes').'</a>',
                                'validate' => 'html',
                                'required' =>  array( 'fes-vendor-suspended-email-toggle', 'equals', array( '1' ) ),
                                'default' => 'Dear {firstname}, 
        Your vendor account has been suspended.',
                            ),
                            array(
                                'id'=> 'fes-vendor-unsuspended-email-toggle',
                                'type' => 'switch',
                                'title' => __( 'Vendor Unsuspended', 'edd_fes' ),
                                'subtitle' => __( 'This email is sent when vendor is unsuspended', 'edd_fes' ),
                                'default'       => 1,
                            ),
                            array(
                                'id' => 'fes-vendor-unsuspended-email',
                                'type' => 'textarea',
                                'title' => __( 'Vendor Unsuspended Email Contents', 'edd_fes' ),
                                'subtitle' => __( 'This setting sets what is in the email body for the vendor unsuspended email.', 'edd_fes' ),
                                'desc' => __( 'A list of shortcodes you can use is ', 'edd_fes' ).'<a href="http://docs.easydigitaldownloads.com/article/338-frontend-submissions-email-configuration" target="_blank" >'.__(' in our documentation','edd_fes').'</a>',
                                'validate' => 'html',
                                'required' =>  array( 'fes-vendor-unsuspended-email-toggle', 'equals', array( '1' ) ),
                                'default' => 'Dear {firstname}, 
        Your vendor account has been reinstated.',
                            ),
                            array(
                                'id'=> 'fes-vendor-new-submission-email-toggle',
                                'type' => 'switch',
                                'title' => __( 'New Submission', 'edd_fes' ),
                                'subtitle' => __( 'This email is sent when vendor submits a new product and the "auto approve submissions" setting is off', 'edd_fes' ),
                                'default'       => 1,
                                'required' =>  array( 'fes-auto-approve-submissions', 'equals', array( '0' ) ),
                            ),
                            array(
                                'id' => 'fes-vendor-new-submission-email',
                                'type' => 'textarea',
                                'title' => __( 'New Submission Email Contents', 'edd_fes' ),
                                'subtitle' => __( 'This setting sets what is in the email body for the new submission email.', 'edd_fes' ),
                                'desc' => __( 'A list of shortcodes you can use is ', 'edd_fes' ).'<a href="http://docs.easydigitaldownloads.com/article/338-frontend-submissions-email-configuration" target="_blank" >'.__(' in our documentation','edd_fes').'</a>',
                                'validate' => 'html',
                                'required' =>  array( 'fes-vendor-new-submission-email-toggle', 'equals', array( '1' ) ),
                                'default' => 'Dear {firstname}, 
        Your submission, {post-title}, has been received.

        It will be reviewed as soon as possible.',
                            ),
                            array(
                                'id'=> 'fes-vendor-submission-approved-email-toggle',
                                'type' => 'switch',
                                'title' => __( 'Submission Approved', 'edd_fes' ),
                                'subtitle' => __( 'This email is sent when a new submitted product is approved and the "auto approve submissions" setting is off ', 'edd_fes' ),
                                'default'       => 1,
                                'required' =>  array( 'fes-auto-approve-submissions', 'equals', array( '0' ) ),
                            ),
                            array(
                                'id' => 'fes-vendor-submission-approved-email',
                                'type' => 'textarea',
                                'title' => __( 'Submission Approved Email Contents', 'edd_fes' ),
                                'subtitle' => __( 'This setting sets what is in the email body for the submission approved email.', 'edd_fes' ),
                                'desc' => __( 'A list of shortcodes you can use is ', 'edd_fes' ).'<a href="http://docs.easydigitaldownloads.com/article/338-frontend-submissions-email-configuration" target="_blank" >'.__(' in our documentation','edd_fes').'</a>',
                                'validate' => 'html',
                                'required' =>  array( 'fes-vendor-submission-approved-email-toggle', 'equals', array( '1' ) ),
                                'default' => 'Dear {firstname}, 
        Your submission, {post-title}, has been approved!',
                            ),
                            array(
                                'id'=> 'fes-vendor-submission-declined-email-toggle',
                                'type' => 'switch',
                                'title' => __( 'Submission Declined', 'edd_fes' ),
                                'subtitle' => __( 'This email is sent when a new submitted product is declined and the "auto approve submissions" setting is off ', 'edd_fes' ),
                                'default'       => 1,
                                'required' =>  array( 'fes-auto-approve-submissions', 'equals', array( '0' ) ),
                            ),
                            array(
                                'id' => 'fes-vendor-submission-declined-email',
                                'type' => 'textarea',
                                'title' => __( 'Submission Declined Email Contents', 'edd_fes' ),
                                'subtitle' => __( 'This setting sets what is in the email body for the submission declined email.', 'edd_fes' ),
                                'desc' => __( 'A list of shortcodes you can use is ', 'edd_fes' ).'<a href="http://docs.easydigitaldownloads.com/article/338-frontend-submissions-email-configuration" target="_blank" >'.__(' in our documentation','edd_fes').'</a>',
                                'validate' => 'html',
                                'required' =>  array( 'fes-vendor-submission-declined-email-toggle', 'equals', array( '1' ) ),
                                'default' => 'Dear {firstname}, 
        Your submission, {post-title}, has been declined.',
                            ),
                            array(
                                'id'=> 'fes-vendor-submission-revoked-email-toggle',
                                'type' => 'switch',
                                'title' => __( 'Submission Revoked', 'edd_fes' ),
                                'subtitle' => __( 'This email is sent when a product is revoked', 'edd_fes' ),
                                'default'       => 1,
                            ),
                            array(
                                'id' => 'fes-vendor-submission-revoked-email',
                                'type' => 'textarea',
                                'title' => __( 'Submission Revoked Email Contents', 'edd_fes' ),
                                'subtitle' => __( 'This setting sets what is in the email body for the submission revoked email.', 'edd_fes' ),
                                'desc' => __( 'A list of shortcodes you can use is ', 'edd_fes' ).'<a href="http://docs.easydigitaldownloads.com/article/338-frontend-submissions-email-configuration" target="_blank" >'.__(' in our documentation','edd_fes').'</a>',
                                'validate' => 'html',
                                'required' =>  array( 'fes-vendor-submission-revoked-email-toggle', 'equals', array( '1' ) ),
                                'default' => 'Dear {firstname}, 
        Your submission, {post-title}, has been revoked.',
                            ),
                )
            );
            $this->sections[] = array(
                'icon' => 'el-icon-bullhorn',
                'title' => __( 'Integrations', 'edd_fes' ),
                'desc' => __( '<p class="description">Settings for FES Integrations</p>', 'edd_fes' ),
                'fields' => array(
                    array(
                        'id' => 'fes-recaptcha-public-key',
                        'type' => 'text',
                        'title' => __( 'reCAPTCHA Site key', 'edd_fes' ),
                        'subtitle' => __( 'You can create your reCAPTCHA keys ', 'edd_fes' ) . '<a href="https://www.google.com/recaptcha/admin#list" target="_blank">'. __( 'here', 'edd_fes' ) . '</a>',
                    ),
                    array(
                        'id' => 'fes-recaptcha-private-key',
                        'type' => 'text',
                        'title' => __( 'reCAPTCHA Secret key', 'edd_fes' ),
                        'subtitle' => __( 'You can create your reCAPTCHA keys ', 'edd_fes' ) . '<a href="https://www.google.com/recaptcha/admin#list" target="_blank">'. __( 'here', 'edd_fes' ) . '</a>',
                    ),
                    array(
                        'id'=> 'fes-login-captcha',
                        'type' => 'switch',
                        'title' => __( 'reCAPTCHA on the login form', 'edd_fes' ),
                        'subtitle' => __( 'If on, creates a reCAPTCHA field on the login form on the vendor dashboard', 'edd_fes' ),
                        'default'       => 0,
                        'required' =>   array(  
                                            array( 'fes-recaptcha-public-key', 'not', '' ),
                                            array( 'fes-recaptcha-private-key', 'not', '' ),
                                    ),
                    ),
                    array(
                        'id'=> 'fes-vendor-contact-captcha',
                        'type' => 'switch',
                        'title' => __( 'reCAPTCHA on the vendor contact form', 'edd_fes' ),
                        'subtitle' => __( 'If on, creates a reCAPTCHA field on the vendor contact form', 'edd_fes' ),
                        'default'       => 0,
                        'required' =>  array(  
                                            array( 'fes-recaptcha-public-key', 'not', '' ),
                                            array( 'fes-recaptcha-private-key', 'not', '' ),
                                    ),
                    ),
                    // Checkout Field Manager integration post 2.3
                    // Commissions override email notification integration post 2.3
                    // QR codes integration post 2.3
                    // Product Support integration post 2.3 (pending Trello vote)
                )
            );
            do_action( 'fes_settings_panel_sections', $this->sections );
        }

        /**
         * All the possible arguments for Redux.
         * For full documentation on arguments, please refer to: https://github.com/ReduxFramework/ReduxFramework/wiki/Arguments
         * */
        public function setArguments() {
            $this->args = array(
                // TYPICAL -> Change these values as you need/desire
                'opt_name' => 'fes_settings', // This is where your data is stored in the database and also becomes your global variable name.
                'display_name' => __( 'Easy Digital Downloads', 'edd_fes' ) . ' ' . fes_plugin_name, // Name that appears at the top of your panel
                'display_version' => fes_plugin_version, // Version that appears at the top of your panel
                'menu_type' => 'submenu', //Specify if the admin menu should appear or not. Options: menu or submenu (Under appearance only)
                'allow_sub_menu' => false, // Show the sections below the admin menu item or not
                'menu_title' => __( 'Settings', 'edd_fes' ),
                'page_title' => __( 'FES Settings', 'edd_fes' ),
                'admin_bar' => false, // Show the panel pages on the admin bar
                'global_variable' => '', // Set a different name for your global variable other than the opt_name
                'dev_mode' => false, // Show the time the page took to load, etc
                'page_priority' => null, // Order where the menu appears in the admin area. If there is any conflict, something will not show. Warning.
                'page_parent' => 'fes-about', // For a full list of options, visit: http://codex.wordpress.org/Function_Reference/add_submenu_page#Parameters
                'page_permissions' => 'manage_shop_settings', // Permissions needed to access the options panel.
                'menu_icon' => '', // Specify a custom URL to an icon
                'last_tab' => '', // Force your panel to always open to a specific tab (by id)
                'page_icon' => 'icon-themes', // Icon displayed in the admin panel next to your menu_title
                'page_slug' => 'fes-settings', // Page slug used to denote the panel
                'save_defaults' => true, // On load save the defaults to DB before user clicks save or not
                'use_cdn' => true,
                'customizer' => false,
                'update_notice' => false,
                'allow_tracking' => false,
                'redux_notice_check' => false,
                'forced_dev_mode_off' => true,
                'default_show' => false, // If true, shows the default value next to each field that is not the default value.
                'default_mark' => '', // What to print by the field's title if the value shown is default. Suggested: *
                // CAREFUL -> These options are for advanced use only
                'transient_time' => 60 * MINUTE_IN_SECONDS,
                'output' => true, // Global shut-off for dynamic CSS output by the framework. Will also disable google fonts output
                'output_tag' => true, // Allows dynamic CSS to be generated for customizer and google fonts, but stops the dynamic CSS from going to the head
                //'domain'              => 'redux-framework', // Translation domain key. Don't change this unless you want to retranslate all of Redux.
                'footer_credit'       => __( 'Thanks for using EDD FES', 'edd_fes' ), // Disable the footer credit of Redux. Please leave if you can help it.
                // FUTURE -> Not in use yet, but reserved or partially implemented. Use at your own risk.
                'database' => '', // possible: options, theme_mods, theme_mods_expanded, transient. Not fully functional, warning!
                'show_import_export' => true, // REMOVE
                'system_info' => false, // REMOVE
                'help_tabs' => array(),
                'help_sidebar' => '', // __( '', $this->args['domain'] );
                'hints' => array(
                    'icon'              => 'icon-question-sign',
                    'icon_position'     => 'right',
                    'icon_color'        => 'lightgray',
                    'icon_size'         => 'normal',

                    'tip_style'         => array(
                        'color'     => 'light',
                        'shadow'    => true,
                        'rounded'   => false,
                        'style'     => '',
                    ),
                    'tip_position'      => array(
                        'my' => 'top left',
                        'at' => 'bottom right',
                    ),
                    'tip_effect' => array(
                        'show' => array(
                            'effect'    => 'slide',
                            'duration'  => '500',
                            'event'     => 'mouseover',
                        ),
                        'hide' => array(
                            'effect'    => 'slide',
                            'duration'  => '500',
                            'event'     => 'click mouseleave',
                        ),
                    ),
                )
            );

            // Panel Intro text -> before the form
            if ( !isset( $this->args['global_variable'] ) || $this->args['global_variable'] !== false ) {
                if ( !empty( $this->args['global_variable'] ) ) {
                    $v = $this->args['global_variable'];
                } else {
                    $v = str_replace( "-", "_", $this->args['opt_name'] );
                }
                $this->args['intro_text'] = __( 'Thanks for using EDD FES', 'edd_fes' );
            }
        }

    }
    global $fes_settings_save;
   $fes_settings_save = new FES_Settings();
}
