<?php
if ( !defined( 'ABSPATH' ) ) {
	exit;
}

class FES_Edit_Download {

	function __construct() {
		add_action( 'wp_dropdown_users', array( $this, 'author_vendor_roles' ), 0, 1 );
		add_action( 'add_meta_boxes', array( $this, 'add_meta_boxes' ) );
		add_action( 'save_post', array( $this, 'save_meta' ), 11, 2 );
		add_action( 'add_meta_boxes', array( &$this, 'change_author_meta_box_title' ) );
	}


	/**
	 * Override the authors selectbox with +vendor roles
	 *
	 * @param html    $output
	 *
	 * @return html
	 */
	public function author_vendor_roles( $output ) {
		global $post;

		if ( empty( $post ) || !is_object( $post ) || is_wp_error( $post ) ) {
			return $output;
		}

		// Return if this isn't an EDD download post type
		if ( $post->post_type != 'download' ) {
			return $output;
		}

		// Return if this isn't the vendor author override dropdown
		if ( !strpos( $output, 'post_author_override' ) ) {
			return $output;
		}

		$args = array(
			'selected' => $post->post_author,
			'id'       => 'post_author_override',
		);

		$output = $this->vendor_selectbox( $args );

		return $output;
	}


	/**
	 * Create a selectbox to display vendor & administrator roles
	 *
	 * @param array   $args
	 *
	 * @return html
	 */
	public function vendor_selectbox( $args ) {
		$default_args = array(
			'placeholder',
			'id',
			'class',
		);

		foreach ( $default_args as $key ) {
			if ( !is_array( $key ) && empty( $args[ $key ] ) ) {
				$args[ $key ] = '';
			} else if ( is_array( $key ) ) {
				foreach ( $key as $val ) {
					$args[ $key ][ $val ] = esc_attr( $args[ $key ][ $val ] );
				}
			}
		}
		extract( $args );

		$roles     = array( 'editor', 'suspended_vendor', 'frontend_vendor', 'administrator', 'contributor', 'shop_manager', 'shop_vendor', 'author' );
		$user_args = array( 'fields' => array( 'ID', 'user_login' ) );

		$output = "<select style='width:200px;' name='$id' id='$id' class='$class' data-placeholder='$placeholder'>\n";
		$output .= "\t<option value=''></option>\n";

		foreach ( $roles as $role ) {

			$new_args           = $user_args;
			$new_args['role'] = $role;
			$users              = get_users( $new_args );

			if ( empty( $users ) ) {
				continue;
			}

			foreach ( (array) $users as $user ) {
				$select = selected( $user->ID, $selected, false );
				$output .= "\t<option value='$user->ID' $select>$user->user_login</option>\n";
			}

		}
		$output .= "</select>";

		// Convert this selectbox with jQuery Chosen
		// todo: Use Select2 when WordPress or EDD adds it to core
		$output .= '<script type="text/javascript">jQuery(function() {jQuery("#' . $id . '").chosen();});</script>';

		return $output;
	}

	public function change_author_meta_box_title() {
		global $wp_meta_boxes;
		$wp_meta_boxes['download']['normal']['core']['authordiv']['title'] = EDD_FES()->helper->get_vendor_constant_name( $plural = false, $uppercase = true );
	}

	public function add_meta_boxes() {
		add_meta_box( 'fes-custom-fields', __( 'Frontend Submissions', 'edd_fes' ) , array( $this, 'render_form' ) , 'download', 'normal', 'high' );
	}

	public function render_form( $type = 'submission', $id = false, $read_only = false, $args = array() ) {
		global $post;
		$form_id = EDD_FES()->helper->get_option( 'fes-submission-form', false );
		if ( !$form_id ) {
			return _e( 'Submission form not set in FES settings' , 'edd_fes' );
		}
		$form = EDD_FES()->helper->get_form_by_id( $form_id, $post->ID );
		$author = false;
		if ( is_object( $post ) ) {
			$author = $post->post_author;
		}
		if ( $author ) {
			$db_user = new FES_DB_Vendors();
			if ( $db_user->exists( 'user_id', $author ) ) {
				$vendor = new FES_Vendor( (int) $author, true );
				$vid = $vendor->id; ?>
				<a href="<?php echo admin_url( "admin.php?page=fes-vendors&view=overview&id=". $vid ."" ); ?>"><?php echo __( 'View', 'edd_fes' ) .  ' ' . EDD_FES()->helper->get_vendor_constant_name( $plural = false, $uppercase = false ) . ' ' . __( 'edit page', 'edd_fes' );?></a><br /><br />
				<?php
			}
		}
		echo $form->render_form_admin();
	}

	public function save_meta( $post_id, $post ) {
		if ( isset( $post->post_type ) && $post->post_type !== 'download' ) {
			return;
		}

		if ( ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) || ( defined( 'DOING_AJAX' ) && DOING_AJAX ) || isset( $_REQUEST['bulk_edit'] ) ) {
			return;
		}

		if ( ! current_user_can( 'edit_product', $post_id ) ) {
			return;
		}

		// Is the user allowed to edit the post or page?
		$form_id = EDD_FES()->helper->get_option( 'fes-submission-form', false );

		if ( current_user_can( 'edit_post', $post_id ) && $form_id ) {
			// Make the FES Form
			$form      = new FES_Submission_Form( $form_id, 'id', $post_id );
			// Save the FES Form
			$form->save_form( $_POST, get_current_user_id() );
		}
	}
}
