<?php
if ( !defined( 'ABSPATH' ) ) {
	exit;
}

add_filter( 'post_updated_messages'   , 'fes_forms_form_updated_message' );
add_action( 'add_meta_boxes_fes-forms', 'fes_forms_add_meta_boxes' );
add_action( 'wp_ajax_fes_formbuilder' , 'fes_forms_ajax_post_add_field' );
add_action( 'save_post'               , 'fes_forms_save_form', 1, 2 );

function fes_forms_form_updated_message( $messages ) {
	$message = array(
		0  => '',
		1  => __( 'Form updated.', 'edd_fes' ) ,
		2  => __( 'Form updated.', 'edd_fes' ) ,
		3  => __( 'Form updated.', 'edd_fes' ) ,
		4  => __( 'Form updated.', 'edd_fes' ) ,
		5  => isset( $_GET['revision'] ) ? sprintf( __( 'Form restored to revision from %s', 'edd_fes' ) , wp_post_revision_title( (int) $_GET['revision'], false ) ) : false,
		6  => __( 'Form updated.', 'edd_fes' ) ,
		7  => __( 'Form updated.', 'edd_fes' ) ,
		8  => __( 'Form updated.', 'edd_fes' ) ,
		9  => __( 'Form updated.', 'edd_fes' ) ,
		10 => __( 'Form updated.', 'edd_fes' )
	);
	$messages['fes-forms'] = $message;
	$messages = apply_filters( 'fes_forms_form_updated_message', $messages );
	return $messages;
}

function fes_forms_add_meta_boxes() {
	$tab = isset( $_REQUEST['tab'] ) ? isset( $_REQUEST['tab'] ) : 'fields';
	$id = get_the_ID();
	if ( empty( $id ) ){
		return;
	}

	if ( $tab == 'fields' ) {
		add_meta_box( 'fes-metabox-editor', __( 'Form Editor', 'edd_fes' ) , 'fes_formbuilder_fields_metabox'      , 'fes-forms', 'normal', 'high' );
		add_meta_box( 'fes-metabox-save'  , __( 'Save', 'edd_fes' )        , 'fes_formbuilder_save'   , 'fes-forms', 'side'  , 'core' );
		$fname = '';
		foreach ( EDD_FES()->load_forms as $name => $class ) {
			$form = new $class( $name, 'name' );
			if ( $form->id == $id ) {
				$fname = $form->title();
				break;
			}
		}
		add_meta_box( 'fes-metabox-fields-specific', __( 'Add '. $fname . ' Form Fields', 'edd_fes' ) , 'fes_formbuilder_sidebar_specfic', 'fes-forms', 'side'  , 'core' );
		add_meta_box( 'fes-metabox-fields-custom', __( 'Add Custom Fields', 'edd_fes' ) , 'fes_formbuilder_sidebar_custom', 'fes-forms', 'side'  , 'core' );
		add_meta_box( 'fes-metabox-fields-extension', __( 'Add Extension Created Fields', 'edd_fes' ) , 'fes_formbuilder_sidebar_extension', 'fes-forms', 'side'  , 'core' );
		remove_meta_box( 'submitdiv', 'fes-forms', 'side' );
		remove_meta_box( 'slugdiv', 'fes-forms', 'normal' );
		do_action( 'fes_add_custom_meta_boxes', $id );
	} else if ( $tab == 'settings' ) {
		add_meta_box( 'fes-metabox-editor', __( 'Form Settings', 'edd_fes' ) , 'fes_formbuilder_settings_metabox'      , 'fes-forms', 'normal', 'high' );
		add_meta_box( 'fes-metabox-save'  , __( 'Save', 'edd_fes' )        , 'fes_formbuilder_save'   , 'fes-forms', 'side'  , 'core' );
	} else if ( $tab == 'notifications' ) {
		add_meta_box( 'fes-metabox-editor', __( 'Form Notifications', 'edd_fes' ) , 'fes_formbuilder_notifications_metabox'      , 'fes-forms', 'normal', 'high' );
		add_meta_box( 'fes-metabox-save'  , __( 'Save', 'edd_fes' )        , 'fes_formbuilder_save'   , 'fes-forms', 'side'  , 'core' );
	} else {
		//do_action( 'fes_register_custom_formbuilder_metaboxes', $tab ); // Do not use until FES 2.4
	}
}

function fes_formbuilder_fields_metabox( $post ) {
	$id = get_the_ID();
	if ( empty( $id ) ){
		return;
	}
	$fname = '';
	$lowerfname = '';
	foreach ( EDD_FES()->load_forms as $name => $class ) {
		$form = new $class( $name, 'name' );
		if ( $form->id == $id ) {
			$fname      = $form->title();
			$lowerfname = strtolower( $fname );
			break;
		}
	}

	$title = __( 'FES '. $fname .' Form Fields', 'edd_fes' );
	$title = apply_filters( 'fes_' . $lowerfname . '_form_formbuilder_fields_metabox_title', $title, $id ); ?>
	<h1><?php echo $title; ?></h1>
	<div class="tab-content">
		<div id="fes-metabox" class="group">
		<?php
		global $post, $pagenow;
		$form = __( 'Your ' . $lowerfname . ' form has no fields', 'edd_fes' );
		$form = apply_filters( 'fes_edit_form_area_no_fields', $form, $id );
		$form_inputs = get_post_meta( $post->ID, 'fes-form', true ) ;
		$form_inputs = isset( $form_inputs ) ? $form_inputs : array(); ?>

		<input type="hidden" name="fes-formbuilder-fields" id="fes-formbuilder-fields" value="<?php echo wp_create_nonce( "fes-formbuilder-fields" ); ?>" />
		<div style="margin-bottom: 10px">
			<button class="button fes-collapse"><?php _e( 'Toggle All Fields Open/Close', 'edd_fes' ); ?></button>
		</div>

		<?php
		if ( empty( $form_inputs ) ) { ?>
			<div class="fes-updated">
			  <p><?php echo $form; ?></p>
			</div>
			<?php } ?>

			<ul id="fes-formbuilder-fields" class="fes-formbuilder-fields unstyled">
				<?php
				if ( $form_inputs ) {
					$count = 0;
					foreach ( $form_inputs as $order => $input_field ) {
						if ( fes_is_key( $input_field['template'], EDD_FES()->load_fields ) ) {
							$class = EDD_FES()->load_fields[ $input_field['template'] ];
							$name  = isset( $input_field['name'] ) ? trim( $input_field['name'] ) : '';
							$class = new $class( $name, $id );
							echo $class->render_formbuilder_field( $count, false );
						} else {
							_fes_deprecated_function( 'Inserting a custom field without using FES Fields API', '2.3' );
							do_action( 'fes_admin_field_' . $input_field['template'], $count, $input_field );
						}
						$count++;
					}
				} ?>
			</ul>
		</div>
	</div>
	<?php
}


function fes_formbuilder_settings_metabox( $post ) {
	$id = get_the_ID();
	if ( empty( $id ) ){
		return;
	}
	$fname = '';
	$lowerfname = '';
	foreach ( EDD_FES()->load_forms as $name => $class ) {
		$form = new $class( $name, 'name' );
		if ( $form->id == $id ) {
			$fname      = $form->title();
			$lowerfname = strtolower( $fname );
			break;
		}
	}

	$title = __( 'FES '. $fname .' Form Settings', 'edd_fes' );
	$title = apply_filters( 'fes_' . $lowerfname . '_form_formbuilder_settings_metabox_title', $title, $id ); ?>
  <h1><?php echo $title; ?></h1>
  <div class="tab-content">
	<div id="fes-metabox" class="group">

	</div>
  </div>
  <?php
}

function fes_formbuilder_notifications_metabox( $post ) {
	$id = get_the_ID();
	if ( empty( $id ) ){
		return;
	}
	$fname = '';
	$lowerfname = '';
	foreach ( EDD_FES()->load_forms as $name => $class ) {
		$form = new $class( $name, 'name' );
		if ( $form->id == $id ) {
			$fname      = $form->title();
			$lowerfname = strtolower( $fname );
			break;
		}
	}

	$title = __( 'FES '. $fname .' Form Notifications', 'edd_fes' );
	$title = apply_filters( 'fes_' . $lowerfname . '_form_formbuilder_notifications_metabox_title', $title, $id ); ?>
	<h1><?php echo $title; ?></h1>
		<div class="tab-content">
			<div id="fes-metabox" class="group">

			</div>
		</div>
	<?php
}

function fes_formbuilder_save() {
	if ( get_the_ID() == EDD_FES()->helper->get_option( 'fes-submission-form', false ) ) {
		?>
		<script>
		(function($) {
			var menu = $( "#toplevel_page_fes-about" );

			menu.removeClass( 'wp-not-current-submenu' );
			menu.addClass( 'wp-has-current-submenu' );

			var link = $( "#toplevel_page_fes-about" ).find( ".toplevel_page_fes-about");

			link.removeClass( 'wp-not-current-submenu' );
			link.addClass( "wp-has-current-submenu" );
			link.addClass( "wp-menu-open" );

			var submenu = menu.find( "li:nth-child(4)" );
			submenu.addClass( "current" );
		})( jQuery );
		</script>
		<?php
	} else if ( get_the_ID() == EDD_FES()->helper->get_option( 'fes-profile-form', false ) ) {
		?>
		<script>
		(function($) {
			var menu = $( "#toplevel_page_fes-about" );

			menu.removeClass( 'wp-not-current-submenu' );
			menu.addClass( 'wp-has-current-submenu' );

			var link = $( "#toplevel_page_fes-about" ).find( ".toplevel_page_fes-about");

			link.removeClass( 'wp-not-current-submenu' );
			link.addClass( "wp-has-current-submenu" );
			link.addClass( "wp-menu-open" );

			var submenu = menu.find( "li:nth-child(6)" );
			submenu.addClass( "current" );
		})( jQuery );
		</script>
		<?php
	} else if ( get_the_ID() == EDD_FES()->helper->get_option( 'fes-registration-form', false ) ) {
		?>
		<script>
		(function($) {
			var menu = $( "#toplevel_page_fes-about" );

			menu.removeClass( 'wp-not-current-submenu' );
			menu.addClass( 'wp-has-current-submenu' );

			var link = $( "#toplevel_page_fes-about" ).find( ".toplevel_page_fes-about");

			link.removeClass( 'wp-not-current-submenu' );
			link.addClass( "wp-has-current-submenu" );
			link.addClass( "wp-menu-open" );

			var submenu = menu.find( "li:nth-child(5)" );
			submenu.addClass( "current" );
		})( jQuery );
		</script>
		<?php
	}


?>
	<div class="submitbox" id="submitpost">
		<div id="minor-publishing">
			<div id="minor-publishing-actions">
				<center>
					<input name="original_publish" type="hidden" id="original_publish" value="<?php esc_attr_e( 'Save' ) ?>" />
					<input name="save" type="submit" class="button button-primary button-large" id="publish" accesskey="p" value="<?php esc_attr_e( 'Save' ) ?>" />
					<span class="spinner"></span>
				</center>
			</div>
			<div class="fes-clear"></div>
		</div>
	</div>
	<?php
}

function fes_formbuilder_sidebar_specfic() { ?>
	<div class="fes-form-buttons">
	<?php
	// foreach fields as field
	// does field support this form? (index "forms" in supports )
	// if yes output button. Name from defaults, label from defaults.
	$id = get_the_ID();
	if ( empty( $id ) ){
		return;
	}
	$fkey  = '';
	$fname = '';
	foreach ( EDD_FES()->load_forms as $name => $class ) {
		$form = new $class( $name, 'name' );
		if ( $form->id == $id ) {
			$fname = $form->title();
			$fkey  = $form->name;
			break;
		}
	}

	$order = array();
	$order[ $fname . ' Form Specific Fields']      = array();
	foreach ( EDD_FES()->load_fields as $fid => $field ) {
		$class = EDD_FES()->load_fields[ $fid ];
		$fieldo = new $class;
		if ( isset( $fieldo->supports['position'] ) && isset( $fieldo->supports['forms'][ $fkey ] ) && $fieldo->supports['forms'][ $fkey ] ) {
			if ( $fieldo->supports['position'] == 'specific' ) {
				$order[ $fname . ' Form Specific Fields'][] = $fieldo;
			}
		}
	}
	$title = esc_attr( __( 'Click to add to the editor', 'edd_fes' ) );
	usort( $order[ $fname . ' Form Specific Fields'], 'fes_field_sort');
	foreach ( $order as $type => $index ) {
		if ( count( $index )  >= 1 ) {
			foreach ( $index as $fielde ) {
				echo '<button class="fes-button button" data-formid="' . get_the_ID() . '" data-name="'.$fielde->supports['template'].'" data-type="'.$fielde->supports['template'].'" title="' . $title . '">'. __( $fielde->supports['title'] , 'edd_fes' ) .'</button>';
			}
		}
		else {
			echo __( 'There are no form specific fields for this form', 'edd_fes' );
		}
	} ?>
  </div>
  <?php
}


function fes_formbuilder_sidebar_custom() { ?>
  <div class="fes-form-buttons">
  <?php
	// foreach fields as field
	// does field support this form? (index "forms" in supports )
	// if yes output button. Name from defaults, label from defaults.
	$id = get_the_ID();
	if ( empty( $id ) ){
		return;
	}
	$fkey  = '';
	$fname = '';
	foreach ( EDD_FES()->load_forms as $name => $class ) {
		$form = new $class( $name, 'name' );
		if ( $form->id == $id ) {
			$fname = $form->title();
			$fkey  = $form->name;
			break;
		}
	}

	$order = array();
	$order['Custom Fields'] = array();

	foreach ( EDD_FES()->load_fields as $fid => $field ) {
		$class = EDD_FES()->load_fields[ $fid ];
		$fieldo = new $class;
		if ( isset( $fieldo->supports['position'] ) && isset( $fieldo->supports['forms'][ $fkey ] ) && $fieldo->supports['forms'][ $fkey ] ) {
			if ( $fieldo->supports['position'] == 'custom' ) {
				$order['Custom Fields'][] = $fieldo;
			}
		}
	}

	$title = esc_attr( __( 'Click to add to the editor', 'edd_fes' ) );
	usort( $order['Custom Fields'], 'fes_field_sort');
	foreach ( $order as $type => $index ) {
		if ( count( $index )  >= 1 ) {
			foreach ( $index as $fielde ) {
				echo '<button class="fes-button button" data-formid="' . get_the_ID() . '" data-name="'.$fielde->supports['template'].'" data-type="'.$fielde->supports['template'].'" title="' . $title . '">'. __( $fielde->supports['title'] , 'edd_fes' ) .'</button>';
			}
		}
		else {
			echo __( 'There are no custom fields for this form', 'edd_fes' );
		}
	} ?>
  </div>
  <?php
}


function fes_formbuilder_sidebar_extension() { ?>
  <div class="fes-form-buttons">
	  <?php
		// foreach fields as field
		// does field support this form? (index "forms" in supports )
		// if yes output button. Name from defaults, label from defaults.
		$id = get_the_ID();
		if ( empty( $id ) ){
			return;
		}
		$fkey  = '';
		$fname = '';
		foreach ( EDD_FES()->load_forms as $name => $class ) {
			$form = new $class( $name, 'name' );
			if ( $form->id == $id ) {
				$fname = $form->title();
				$fkey  = $form->name;
				break;
			}
		}

		$order = array();
		$order['Extension Created Fields'] = array();
		foreach ( EDD_FES()->load_fields as $fid => $field ) {
			$class = EDD_FES()->load_fields[ $fid ];
			$fieldo = new $class;
			if ( isset( $fieldo->supports['position'] ) && isset( $fieldo->supports['forms'][ $fkey ] ) && $fieldo->supports['forms'][ $fkey ] ) {
				if ( $fieldo->supports['position'] == 'extension' ) {
					$order['Extension Created Fields'][] = $fieldo;
				}
			}
		}

		$title = esc_attr( __( 'Click to add to the editor', 'edd_fes' ) );
		usort( $order['Extension Created Fields'], 'fes_field_sort');
		foreach ( $order as $type => $index ) {
			if ( count( $index )  >= 1 ) {
				foreach ( $index as $fielde ) {
					echo '<button class="fes-button button" data-formid="' . get_the_ID() . '" data-name="'.$fielde->supports['template'].'" data-type="'.$fielde->supports['template'].'" title="' . $title . '">'. __( $fielde->supports['title'] , 'edd_fes' ) .'</button>';
				}
			}
			else {
				if ( get_the_ID() == EDD_FES()->helper->get_option( 'fes-submission-form', false ) && !has_action( 'fes_custom_post_button' ) ) {
					echo __( 'There are no extension fields for this form', 'edd_fes' );
				} else if ( get_the_ID() == EDD_FES()->helper->get_option( 'fes-profile-form', false ) && !has_action( 'fes_custom_profile_button' ) ) {
					echo __( 'There are no extension fields for this form', 'edd_fes' );
				} else if ( get_the_ID() == EDD_FES()->helper->get_option( 'fes-registration-form', false ) && !has_action( 'fes_custom_registration_button' ) ) {
					echo __( 'There are no extension fields for this form', 'edd_fes' );
				}
			}
		}

		if ( get_the_ID() == EDD_FES()->helper->get_option( 'fes-submission-form', false ) ) {
			do_action( 'fes_custom_post_button', $title ); // note: don't use these. Register a FES field
		} else if ( get_the_ID() == EDD_FES()->helper->get_option( 'fes-profile-form', false ) ) {
			do_action( 'fes_custom_profile_button' ); // note: don't use these. Register a FES field
		} else if ( get_the_ID() == EDD_FES()->helper->get_option( 'fes-registration-form', false ) ) {
			do_action( 'fes_custom_registration_button' ); // note: don't use these. Register a FES field
		} ?>
	</div>
	<?php
}

function fes_forms_save_form( $post_id, $post ) {
	if ( empty( $post_id ) || $post_id < 1 || !is_object( $post ) || $post->post_type !== 'fes-forms' ) {
		return;
	}

	if ( isset( $_POST['fes-formbuilder-fields'] ) && wp_verify_nonce( $_POST['fes-formbuilder-fields'], 'fes-formbuilder-fields' ) ) {
		$values = $_POST['fes_input'];
		foreach ( EDD_FES()->load_forms as $name => $class ) {
			$form = new $class( $name, 'name' );
			if ( $form->id == $post->ID ) {
				$return = false;
				$return = $form->save_formbuilder_fields( $post->ID, $values );
				if ( $return ) {
					return $return;
				}
				break;
			}
		}
	} else if ( isset( $_POST['fes-formbuilder-notifications'] ) && wp_verify_nonce( $_POST['fes-formbuilder-notifications'], 'fes-formbuilder-notifications' ) ) {
			$values = $_POST['fes_notifications'];
			foreach ( EDD_FES()->load_forms as $name => $class ) {
				$form = new $class( $name, 'name' );
				if ( $form->id == $post->ID ) {
					$return = false;
					$return = $form->save_formbuilder_notifications( $post->ID, $values );
					if ( $return ) {
						return $return;
					}
					break;
				}
			}
	} else if ( isset( $_POST['fes-formbuilder-settings'] ) && wp_verify_nonce( $_POST['fes-formbuilder-settings'], 'fes-formbuilder-settings' ) ) {
		$values = $_POST['fes_settings'];
		foreach ( EDD_FES()->load_forms as $name => $class ) {
			$form = new $class( $name, 'name' );
			if ( $form->id == $post->ID ) {
				$return = false;
				$return = $form->save_formbuilder_settings( $post->ID, $values );
				if ( $return ) {
					return $return;
				}
				break;
			}
		}
	} else {
		$custom_tab = apply_filters( 'fes_is_custom_formbuilder_tab', false, $post->ID );
		if ( $custom_tab ) {
			do_action( 'fes_custom_formbuilder_tab_save', $post->ID );
		} else {
			return $post->ID;
		}
	}
}

function fes_forms_ajax_post_add_field() {
	if ( !isset( $_POST['action'] ) || $_POST['action'] !== 'fes_formbuilder' ) {
		exit;
	}
	$name     = isset( $_POST['name'] )  ? $_POST['name'] : '' ;
	$field_id = isset( $_POST['order'] ) ? $_POST['order'] : 0 ;
	$id       = isset( $_POST['id'] )    ? $_POST['id'] : 0 ;

	if ( fes_is_key( $name, EDD_FES()->load_fields ) ) {
		$class = EDD_FES()->load_fields[ $name ];
		$class = new $class( '', $id );
		echo $class->render_formbuilder_field( $field_id, true );
	} else {
		do_action( 'fes_admin_field_' . $name, $field_id );
	}
	exit;
}

function fes_field_sort( $a,$b ) {
    return strcmp($a->supports['title'],$b->supports['title']);
}
