<?php
class FES_Vendor_Table extends WP_List_Table {

	/**
	 * Number of results to show per page
	 *
	 * @since       2.3
	 * @var         int
	 */
	public $per_page = 10;

	/**
	 * Total number of vendors for current view
	 *
	 * @since       2.3
	 * @var         int
	 */
	public $total = 0;


	function __construct() {
		global $status, $page;
		//Set parent defaults
		parent::__construct( array(
			'singular'  => EDD_FES()->helper->get_vendor_constant_name( $plural = false, $uppercase = false ),     //singular name of the listed records
			'plural'    => EDD_FES()->helper->get_vendor_constant_name( $plural = true, $uppercase = false ),    //plural name of the listed records
			'ajax'      => false             //does this table support ajax?
		) );
	}

	/**
	 * Show the search field
	 *
	 * @since 2.3
	 * @access public
	 *
	 * @param string  $text     Label for the search box
	 * @param string  $input_id ID of the search box
	 *
	 * @return void
	 */
	function search_box( $text, $input_id ) {
		$input_id = $input_id . '-search-input';

		if ( ! empty( $_REQUEST['orderby'] ) ) {
			echo '<input type="hidden" name="orderby" value="' . esc_attr( $_REQUEST['orderby'] ) . '" />';
		}
		if ( ! empty( $_REQUEST['order'] ) ) {
			echo '<input type="hidden" name="order" value="' . esc_attr( $_REQUEST['order'] ) . '" />';
		} ?>
		<p class="search-box">
			<label class="screen-reader-text" for="<?php echo $input_id ?>"><?php echo $text; ?>:</label>
			<input type="search" id="<?php echo $input_id ?>" name="s" value="<?php _admin_search_query(); ?>" />
			<?php submit_button( $text, 'button', false, false, array( 'ID' => 'search-submit' ) ); ?>
		</p>
		<?php
	}


	function column_default( $vendor, $column_name ) {
		$data = '';
		switch ( $column_name ) {
		case 'name':
			$data .= '<a href="'.admin_url( 'admin.php?page=fes-vendors&view=overview&id='.$vendor['id'] ).'">'.$vendor['name'].' ('.$vendor['username'].')</a>';
			if ( EDD_FES()->vendors->user_is_admin() ) {
				$admin_actions = array();
				$data .= '<br />';
				$vendor_id = $vendor['id'];
				if ( EDD_FES()->vendors->vendor_is_status( 'approved', $vendor_id ) ) {
					$admin_actions['view']   = array(
						'action' => 'view',
						'name' => __( 'View', 'edd_fes' ),
						'url' => admin_url( 'admin.php?page=fes-vendors&view=overview&id='.$vendor['id'] )
					);
					$admin_actions['revoke'] = array(
						'action' => 'revoked',
						'name' => sprintf( __( 'Revoke (and delete all %s of)', 'edd_fes' ), EDD_FES()->helper->get_product_constant_name( $plural = true, $uppercase = false ) ),
						'url' => '#'
					);
					$admin_actions['suspend'] = array(
						'action' => 'suspended',
						'name' => __( 'Suspend', 'edd_fes' ),
						'url' => '#'
					);
				} else if ( EDD_FES()->vendors->vendor_is_status( 'pending', $vendor_id ) ) {
					$admin_actions['view']   = array(
						'action' => 'view',
						'name' => __( 'View', 'edd_fes' ),
						'url' => admin_url( 'admin.php?page=fes-vendors&view=overview&id='.$vendor['id'] .'&action=edit' )
					);
					$admin_actions['approve'] = array(
						'action' => 'approved',
						'name' => __( 'Approve', 'edd_fes' ),
						'url' => '#'
					);
					$admin_actions['decline'] = array(
						'action' => 'declined',
						'name' => __( 'Decline', 'edd_fes' ),
						'url' => '#'
					);
				} else if ( EDD_FES()->vendors->vendor_is_status( 'suspended', $vendor_id ) ) {
					$admin_actions['view']   = array(
						'action' => 'view',
						'name' => __( 'View', 'edd_fes' ),
						'url' => admin_url( 'admin.php?page=fes-vendors&view=overview&id='.$vendor['id'] .'&action=edit' )
					);
					$admin_actions['revoke'] = array(
						'action' => 'revoked',
						'name' => sprintf( __( 'Revoke (and delete all %s of)', 'edd_fes' ), EDD_FES()->helper->get_product_constant_name( $plural = true, $uppercase = false ) ),
						'url' => '#'
					);
					$admin_actions['unsuspend'] = array(
						'action' => 'unsuspended',
						'name' => __( 'Unsuspend', 'edd_fes' ),
						'url' => '#'
					);
				} else {
					// not a vendor!
				}

				$admin_actions = apply_filters( 'fes_admin_actions', $admin_actions, $vendor );
				foreach ( $admin_actions as $action ) {
					$image = isset( $action['image_url'] ) ? $action['image_url'] : fes_plugin_url . 'assets/img/icons/' . $action['action'] . '.png';
					$class = $action['action'] === 'view' ? '' : 'vendor-change-status';
					$data .= sprintf( '<a class="button tips %s" data-vendor="%d" data-status="%s" data-nstatus="%s" href="%s" data-tip="%s"><img src="%s" alt="%s" width="14" /></a>', $class, (int) esc_attr( $vendor['user_id'] ), esc_attr( $action['action'] ), esc_attr( $action['name'] ), esc_url( $action['url'] ), esc_attr( $action['name'] ), esc_attr( $image ), esc_attr( $action['name'] ) );
				}
			}
			break;
		case 'products':
			$data = $vendor['product_count'];
			break;
		case 'status':
			if ( $vendor['status'] == 'pending' ) {
				$data = '<span class="download-status pending-review">' . __( 'Pending', 'edd_fes' ) . '</span>';
			} else if ( $vendor['status'] == 'approved' ) {
					$data = '<span class="download-status published">' . __( 'Approved', 'edd_fes' ) . '</span>';
				} else if ( $vendor['status'] == 'suspended' ) {
					$data = '<span class="download-status future">' . __( 'Suspended', 'edd_fes' ) . '</span>';
				} else {
				$data = __( 'Unknown Column', 'edd_fes' );
			}
			break;
		case 'date':
			$data = date_i18n( get_option( 'date_format' ), strtotime( $vendor['date_created'] ) );
			break;
		case 'sales_value':
			$data = edd_currency_filter( edd_format_amount( $vendor['sales_value'] ) );
			break;
		case 'sales_count':
			$data = $vendor['sales_count'];
			break;
		default:
			return print_r( $vendor, true ); //Show the whole array for troubleshooting purposes
			break;
		}
		return apply_filters( 'fes_vendor_table_value_' . $column_name, $data );
	}

	function column_cb( $vendor ) {
		return sprintf(
			'<input type="checkbox" name="%1$s[]" value="%2$s" />',
			/*$1%s*/ $this->_args['singular'],
			/*$2%s*/ $vendor['user_id']
		);
	}


	function get_columns() {
		$columns = array(
			'cb'          => '<input type="checkbox" />', //Render a checkbox instead of text
			'name'        => __( 'Name (username)', 'edd_fes' ),
			'status'      => __( 'Status', 'edd_fes' ),
			'products'    => sprintf( _x( 'Number of %s', 'FES plural uppercase setting for download', 'edd_fes' ), EDD_FES()->helper->get_product_constant_name( $plural = true, $uppercase = true ) ),
			'sales_count' => __( 'Sales Count', 'edd_fes' ),
			'sales_value' => __( 'Sales Value', 'edd_fes' ),
			'date'        => sprintf( _x( '%s Since', 'FES plural uppercase setting for download', 'edd_fes' ), EDD_FES()->helper->get_vendor_constant_name( $plural = false, $uppercase = true ) ),
		);
		return $columns;
	}

	function get_views() {
		$base = admin_url( 'admin.php?page=fes-vendors' );
		$current = isset( $_GET['view'] ) ? sanitize_text_field( $_GET['view'] ) : 'all';
		$views = array(
			'all'       => sprintf( '<a href="%s"%s>%s <span class="count">(%s)</span></a>', add_query_arg( 'view', 'all', $base ), $current === 'all' ? ' class="current"' : '', __( 'All', 'edd_fes' ), $this->get_count( array( 'approved', 'suspended', 'pending' ) ) ),
			'pending'   => sprintf( '<a href="%s"%s>%s <span class="count">(%s)</span></a>', add_query_arg( 'view', 'pending', $base ), $current === 'pending' ? ' class="current"' : '', __( 'Pending', 'edd_fes' ), $this->get_count( array( 'pending' ) ) ),
			'approved'  => sprintf( '<a href="%s"%s>%s <span class="count">(%s)</span></a>', add_query_arg( 'view', 'approved', $base ), $current === 'approved' ? ' class="current"' : '', __( 'Approved', 'edd_fes' ), $this->get_count( array( 'approved' ) ) ),
			'suspended' => sprintf( '<a href="%s"%s>%s <span class="count">(%s)</span></a>', add_query_arg( 'view', 'suspended', $base ), $current === 'suspended' ? ' class="current"' : '', __( 'Suspended', 'edd_fes' ), $this->get_count( array( 'suspended' ) ) )
		);
		return $views;
	}


	function get_bulk_actions() {
		$actions = array(
			'approved'   => sprintf( __( 'Approve %s' , 'edd_fes' ), EDD_FES()->helper->get_vendor_constant_name( $plural = true, $uppercase = true ) ),
			'declined'   => sprintf( __( 'Decline %s' , 'edd_fes' ), EDD_FES()->helper->get_vendor_constant_name( $plural = true, $uppercase = true ) ),
			'revoked'    => sprintf( __( 'Revoke %s' , 'edd_fes' ), EDD_FES()->helper->get_vendor_constant_name( $plural = true, $uppercase = true ) ),
			'suspended'   => sprintf( __( 'Suspend %s' , 'edd_fes' ), EDD_FES()->helper->get_vendor_constant_name( $plural = true, $uppercase = true ) ),
			'unsuspended' => sprintf( __( 'Unsuspend %s' , 'edd_fes' ), EDD_FES()->helper->get_vendor_constant_name( $plural = true, $uppercase = true ) ),
		);
		return $actions;
	}


	/**
	 * Retrieve the current page number
	 *
	 * @access      private
	 * @since       1.7
	 * @return      int
	 */
	function get_paged() {
		return isset( $_GET['paged'] ) ? absint( $_GET['paged'] ) : 1;
	}

	/**
	 * Retrieves the search query string
	 *
	 * @access public
	 * @since 1.7
	 * @return mixed string If search is present, false otherwise
	 */
	function get_search() {
		return ! empty( $_GET['s'] ) ? urldecode( trim( $_GET['s'] ) ) : false;
	}

	/**
	 * Retrieve the offset based on the current page number
	 *
	 * @access      private
	 * @since       2.2.6
	 * @return      int
	 */
	function get_offset() {
		$page = $this->get_paged();
		return $this->per_page * ( $page - 1 );
	}

	function get_vendors() {
		global $wpdb;

		$data    = array();
		$paged   = $this->get_paged();
		$offset  = $this->per_page * ( $paged - 1 );
		$search  = $this->get_search();
		$order   = isset( $_GET['order'] )   ? sanitize_text_field( $_GET['order'] )   : 'DESC';
		$orderby = isset( $_GET['orderby'] ) ? sanitize_text_field( $_GET['orderby'] ) : 'id';

		$args    = array(
			'number'  => $this->per_page,
			'offset'  => $offset,
			'order'   => $order,
			'orderby' => $orderby
		);

		if ( strpos( $search, ':' ) !== FALSE ) {
			global $wpdb;
			$pieces = explode( " ", $search );
			$user_by_meta = $wpdb->get_var(
				$wpdb->prepare( "
					SELECT user_id FROM $wpdb->usermeta
					WHERE meta_key = %s
					AND meta_value = %s
					LIMIT 1",
					sanitize_text_field( $pieces[0] ),
					sanitize_text_field( $pieces[1] )
				)
			);
			if ( $user_by_meta ) {
				$args['user_id']    = (int) $user_by_meta;
			}
		} else if ( is_email( $search ) ) {
				global $wpdb;
				$user_by_commissions = $wpdb->get_var(
					$wpdb->prepare( "
					SELECT user_id FROM $wpdb->usermeta
					WHERE meta_key = %s
					AND meta_value = %s
					LIMIT 1",
						'eddc_user_paypal',
						sanitize_email( $search )
					)
				);
				if ( $user_by_commissions ) {
					$args['user_id']    = (int) $user_by_commissions;
				}
				else {
					$args['email'] = $search;
				}
			} 
		else if ( is_numeric( $search ) ) {
			$args['user_id']    = $search;
		} else {
			$vendor = new FES_Vendor( $search );
			if ( $vendor && is_object( $vendor ) && $vendor->exists ) {
				$args['username']  = $search;
			} else {
				$args['name']  = $search;
			}
		}

		if ( isset( $_GET['view'] ) && $_GET['view'] !== 'all' ) {
			$args['status'] = $_GET['view'];
		}

		$vendors = new FES_DB_Vendors();
		$vendors = $vendors->get_vendors( $args );

		if ( $vendors ) {

			foreach ( $vendors as $vendor ) {

				$user_id = ! empty( $vendor->user_id ) ? intval( $vendor->user_id ) : 0;

				$data[] = array(
					'id'            => $vendor->id,
					'user_id'       => $user_id,
					'username'      => $vendor->username,
					'name'          => $vendor->name,
					'email'         => $vendor->email,
					'status'        => $vendor->status,
					'product_count' => $vendor->product_count,
					'sales_count'   => $vendor->sales_count,
					'sales_value'   => $vendor->sales_value,
					'date_created'  => $vendor->date_created,
				);
			}
		}
		return $data;
	}

	function pagination_count() {
		global $wpdb;

		$data    = array();
		$search  = $this->get_search();

		if ( strpos( $search, ':' ) !== FALSE ) {
			global $wpdb;
			$pieces = explode( " ", $search );
			$user_by_meta = $wpdb->get_var(
				$wpdb->prepare( "
					SELECT user_id FROM $wpdb->usermeta
					WHERE meta_key = %s
					AND meta_value = %s
					LIMIT 1",
					sanitize_text_field( $pieces[0] ),
					sanitize_text_field( $pieces[1] )
				)
			);
			if ( $user_by_meta ) {
				$args['user_id']    = (int) $user_by_meta;
			}
		} else if ( is_email( $search ) ) {
				global $wpdb;
				$user_by_commissions = $wpdb->get_var(
					$wpdb->prepare( "
					SELECT user_id FROM $wpdb->usermeta
					WHERE meta_key = %s
					AND meta_value = %s
					LIMIT 1",
						'eddc_user_paypal',
						sanitize_email( $search )
					)
				);
				if ( $user_by_commissions ) {
					$args['user_id']    = (int) $user_by_commissions;
				}
				else {
					$args['email'] = $search;
				}
			} 
		else if ( is_numeric( $search ) ) {
			$args['user_id']    = $search;
		} else {
			$vendor = new FES_Vendor( $search );
			if ( $vendor && is_object( $vendor ) && $vendor->exists ) {
				$args['username']  = $search;
			} else {
				$args['name']  = $search;
			}
		}

		if ( isset( $_GET['view'] ) && $_GET['view'] !== 'all' ) {
			$args['status'] = $_GET['view'];
		}

		$vendors = new FES_DB_Vendors();
		$vendors = $vendors->count( $args );
		return $vendors;
	}	

	function get_count( $roles ) {
		$vendors = new FES_DB_Vendors();

		$total = 0;
		foreach ( $roles as $role ) {
			$total = $total + $vendors->count( array( 'status' => $role ) );
		}

		return $total;
	}

	function get_sortable_columns() {
		$sortable_columns = array(
			'name'          => array( 'name',           true  ),     // true means it's already sorted
			'status'        => array( 'status',         false ),
			'products'      => array( 'product_count',  false ),
			'sales_count'   => array( 'sales_count',    false ),
			'sales_value'   => array( 'sales_value',    false ),
			'date'          => array( 'date',           false ),
		);
		return $sortable_columns;
	}

	/**
	 *
	 *
	 * @uses $this->_column_headers
	 * @uses $this->items
	 * @uses $this->get_columns()
	 * @uses $this->get_sortable_columns()
	 * @uses $this->get_pagenum()
	 * @uses $this->set_pagination_args()
	 * */
	function prepare_items() {

		$columns = $this->get_columns();
		$hidden = array(); // no hidden columns

		$sortable = $this->get_sortable_columns();

		$this->_column_headers = array( $columns, $hidden, $sortable );

		$current_page = $this->get_pagenum();

		$this->items = $this->get_vendors();
		$this->total = $this->pagination_count();
		$this->set_pagination_args( array(
			'total_items' => $this->total,
			'per_page'    => $this->per_page,
			'total_pages' => ceil( $this->total / $this->per_page )
		) );
	}
}
