<?php
class FES_Helpers {
	function __construct() {
		add_shortcode( 'fes_form_display_fields', array( $this,  'display_fields_shortcode' ) );
	}

	// Gets FES setting for the given key. If not set, returns the default
	public function get_option( $key = '', $default = false ) {
		global $fes_settings;
		$value = isset( $fes_settings[ $key ] ) ? $fes_settings[ $key ] : $default;
		$value = apply_filters( 'fes_get_option', $value, $key, $default );
		return apply_filters( 'fes_get_option_' . $key, $value, $key, $default );
	}

	// Sets FES setting for the given key. If not set, returns the default
	public function set_option( $key, $value ) {
		global $fes_settings_save;
		$fes_settings_save->ReduxFramework->set( $key, $value );
	}

	public function get_form_name_by_id( $id ){
		$name = get_post_meta( $id, 'fes-form-name', true );
		if ( !$name ){
			if ( EDD_FES()->helper->get_option( 'fes-submission-form', false ) == $id ) {
				$name = 'submission';
			} else if ( EDD_FES()->helper->get_option( 'fes-profile-form', false ) == $id  ) {
				$name = 'profile';
			} else if ( EDD_FES()->helper->get_option( 'fes-registration-form', false ) == $id  ) {
				$name = 'registration';
			} else if ( EDD_FES()->helper->get_option( 'fes-login-form', false ) == $id  ) {
				$name = 'login';
			} else if ( EDD_FES()->helper->get_option( 'fes-vendor-contact-form', false ) == $id  ) {
				$name = 'vendor-contact';
			} else{
				$name = apply_filters( 'fes_get_form_name_by_id', $name, $id );
			}
		}
		return $name;
	}

	public function get_form_type_by_id( $id ){
		$type = get_post_meta( $id, 'fes-form-type', true );
		if ( !$type ){
			if ( EDD_FES()->helper->get_option( 'fes-submission-form', false ) == $id ) {
				$type = 'post';
			} else if ( EDD_FES()->helper->get_option( 'fes-profile-form', false ) == $id  ) {
				$type = 'user';
			} else if ( EDD_FES()->helper->get_option( 'fes-registration-form', false ) == $id  ) {
				$type = 'user';
			} else if ( EDD_FES()->helper->get_option( 'fes-login-form', false ) == $id  ) {
				$type = 'custom';
			} else if ( EDD_FES()->helper->get_option( 'fes-vendor-contact-form', false ) == $id  ) {
				$type = 'custom';
			} else{
				$type = apply_filters( 'fes_get_form_type_by_id', $type, $id );
			}
		}
		return $type;
	}

	public function get_form_class_by_id( $id ){
		$class = get_post_meta( $id, 'fes-form-class', false );
		if ( !$class ){
			if ( EDD_FES()->helper->get_option( 'fes-submission-form', false ) == $id ) {
				$class = 'FES_Submission_Form';
			} else if ( EDD_FES()->helper->get_option( 'fes-profile-form', false ) == $id  ) {
				$class = 'FES_Profile_Form';
			} else if ( EDD_FES()->helper->get_option( 'fes-registration-form', false ) == $id  ) {
				$class = 'FES_Registration_Form';
			} else if ( EDD_FES()->helper->get_option( 'fes-login-form', false ) == $id  ) {
				$class = 'FES_Login_Form';
			} else if ( EDD_FES()->helper->get_option( 'fes-vendor-contact-form', false ) == $id  ) {
				$class = 'FES_Vendor_Contact_Form';
			} else{
				 $class = apply_filters( 'fes_get_form_class_by_id', $class, $id );
			}
		}
		return $class;
	}

	public function get_form_id_by_name( $name ){
		return EDD_FES()->helper->get_option( 'fes-' . $name . '-form', true );
	}

	public function get_form_class_by_name( $name ){
		if ( fes_is_key( $name, EDD_FES()->load_forms ) ){
			return EDD_FES()->load_forms[ $name ];
		} else {
			$class = apply_filters( 'fes_get_form_class_by_name', false, $name );
			return $class;
		}
	}

	public function get_form_by_name( $name, $save_to = false ){
		$class = get_post_meta( EDD_FES()->helper->get_form_id_by_name( $name ), 'fes-form-class', true );
		if ( $class ){
			$form = new $class( EDD_FES()->helper->get_form_id_by_name( $name ), 'id', $save_to );
			return $form;
		} else {
			$form = apply_filters( 'fes_get_form_by_name', false, $name, $save_to );
			return $form;
		}
	}

	public function get_form_by_id( $id = 0, $save_to = false ){
		$class = get_post_meta( $id, 'fes-form-class', true );
		$form = new $class( $id, 'id', $save_to );
		return $form;
	}

	public function is_fes_form( $id = 0 ){
		$found = false;
		$form = get_post_meta( $id, 'fes-form-class', true );
		if ( !empty( $form ) ) {
			$found = true;
		}
		return $found;
	}

	public function get_form_name_by_class( $class ){
		if ( 'submission' == $class ) {
			$class = 'FES_Submission_Form';
		} else if ( 'profile' == $class  ) {
			$class = 'FES_Profile_Form';
		} else if ( 'registration' == $class  ) {
			$class = 'FES_Registration_Form';
		} else if ( 'login' == $class  ) {
			$class = 'FES_Login_Form';
		} else if ( 'vendor-contact' == $class  ) {
			$class = 'FES_Vendor_Contact_Form';
		} else{
			 $class = apply_filters( 'fes_get_form_name_by_class', $name, $class );
		}

		return $class;
	}

	public function get_field_class_by_name( $name ){
		if ( fes_is_key( $name, EDD_FES()->load_fields ) ){
			return EDD_FES()->load_fields[ $name ];
		} else {
			$class = apply_filters( 'fes_get_form_class_by_name', false, $name );
			return $class;
		}
	}

	public function display_fields_shortcode( $atts ){
		$a = shortcode_atts( array(
			'form_id' => -2,
			'save_id' => -2,
		), $atts );

		$form = EDD_FES()->helper->get_form_by_id( $a['form_id'], $a['save_id'] );
		if ( $form && is_object( $form ) ){
			return $form->display_fields();
		}
	}

	public function get_product_constant_name( $plural = false, $uppercase = true ) {
		$constant = EDD_FES()->helper->get_option( 'fes-product-constant', '' );
		// Products
		if ( $plural && $uppercase ) {
			$constant = ( isset( $constant ) && $constant != '' ) ? ucfirst( $constant ) . 's' : __( 'Products', 'edd_fes' );
			$constant = apply_filters( 'fes_product_constant_plural_uppercase', $constant );
		}
		// products
		else if ( $plural ) {
			$constant = ( isset( $constant ) && $constant != '' ) ? $constant . 's' : __( 'products', 'edd_fes' );
			$constant = apply_filters( 'fes_product_constant_plural_lowercase', $constant );
		}
		// Product
		else if ( !$plural && $uppercase ) {
			$constant = ( isset( $constant ) && $constant != '' ) ? ucfirst( $constant ) : __( 'Product', 'edd_fes' );
			$constant = apply_filters( 'fes_product_constant_singular_uppercase', $constant );
		}
		// product
		else {
			$constant = ( isset( $constant ) && $constant != '' ) ? $constant : __( 'product', 'edd_fes' );
			$constant = apply_filters( 'fes_product_constant_singular_lowercase', $constant );
		}
		return apply_filters ( 'fes_product_constant', $constant, $plural, $uppercase );
	}

	public function get_vendor_constant_name( $plural = false, $uppercase = true ) {
		$constant = EDD_FES()->helper->get_option( 'fes-vendor-constant', '' );
		// Vendors
		if ( $plural && $uppercase ) {
			$constant = ( isset( $constant ) && $constant != '' ) ? ucfirst( $constant ) . 's' : __( 'Vendors', 'edd_fes' );
			$constant = apply_filters( 'fes_vendor_constant_plural_uppercase', $constant );
		}
		// vendors
		else if ( $plural ) {
			$constant = ( isset( $constant ) && $constant != '' ) ? $constant . 's' : __( 'vendors', 'edd_fes' );
			$constant = apply_filters( 'fes_vendor_constant_plural_lowercase', $constant );
		}
		// Vendor
		else if ( !$plural && $uppercase ) {
			$constant = ( isset( $constant ) && $constant != '' ) ? ucfirst( $constant ) : __( 'Vendor', 'edd_fes' );
			$constant = apply_filters( 'fes_vendor_constant_singular_uppercase', $constant );
		}
		// vendor
		else {
			$constant = ( isset( $constant ) && $constant != '' ) ? $constant : __( 'vendor', 'edd_fes' );
			$constant = apply_filters( 'fes_vendor_constant_singular_lowercase', $constant );
		}
		return apply_filters ( 'fes_vendor_constant', $constant, $plural, $uppercase );
	}

	public function get_user_meta( $name, $user_id, $type = 'normal'  ){
		_fes_deprecated_function( 'EDD_FES()->helpers->get_user_meta', '2.1', 'get_user_meta()' );
		if ( empty( $name ) || empty( $user_id ) ) {
			return;
		}

		if ( $type == 'image' || $type == 'file' ) {
			$images = get_user_meta( $user_id, $name );

			if ( $images ) {
				$html = '';
				if ( isset( $images[0] ) && is_array( $images[0] ) ){
					$images = $images[0];
				}
				foreach ($images as $attachment_id ) {
					if ( $type == 'image' ) {
						$thumb = wp_get_attachment_image( $attachment_id, $size );
					} else {
						$thumb = get_post_field( 'post_title', $attachment_id );
					}

					$full_size = wp_get_attachment_url( $attachment_id );
					$html .= sprintf( '<a href="%s">%s</a> ', $full_size, $thumb );
				}
				return $html;
			}
		} elseif ( $type == 'repeat' ) {
			return implode( '; ', get_user_meta( $user_id, $name ) );
		} else {
			return implode( ', ', get_user_meta( $user_id, $name ) );
		}

	}

	public function get_post_meta( $name, $post_id, $type = 'normal' ){
		_fes_deprecated_function( 'EDD_FES()->helpers->get_post_meta', '2.1', 'get_post_meta()' );
		if ( empty( $name ) || empty( $post_id ) ) {
			return;
		}

		$post = get_post( $post_id );

		if ( $type == 'image' || $type == 'file' ) {
			$images = get_post_meta( $post->ID, $name );

			if ( $images ) {
				$html = '';
				if ( isset( $images[0] ) && is_array( $images[0] ) ){
					$images = $images[0];
				}
				foreach ($images as $attachment_id ) {
					if ( $type == 'image' ) {
						$thumb = wp_get_attachment_image( $attachment_id );
					} else {
						$thumb = get_post_field( 'post_title', $attachment_id );
					}

					$full_size = wp_get_attachment_url( $attachment_id );
					$html .= sprintf( '<a href="%s">%s</a> ', $full_size, $thumb );
				}
				return $html;
			}
		} elseif ( $type == 'repeat' ) {
			return implode( '; ', get_post_meta( $post->ID, $name ) );
		} else {
			return implode( ', ', get_post_meta( $post->ID, $name ) );
		}
	}
}
