<?php
if ( !defined( 'ABSPATH' ) ) {
	exit;
}

class FES_Vendors {
	function __construct() {
		add_action( 'admin_init', array( $this, 'prevent_admin_access' ), 1000 );
		add_filter( 'show_admin_bar' , array( $this, 'hide_admin_bar' ), 99999 );
		add_filter( 'edd_user_can_view_receipt', array( $this, 'vendor_can_view_receipt' ), 10, 2 );
		add_filter( 'edd_user_can_view_receipt_item', array( $this, 'vendor_can_view_receipt_item' ), 10, 2 );
		add_action( 'edd_complete_purchase',  array( $this, 'increment_vendor_earnings' ), 10, 1 );
		add_action( 'edd_payment_deleted',  array( $this, 'decrement_vendor_earnings' ), 10, 1 );
		add_action( 'post_updated', array( $this, 'change_author_check' ), 10, 3 );
		add_action( 'transition_post_status', array( $this, 'post_status_transition' ), 10, 3 );
	}

	public function change_author_check( $post_id, $post_after, $post_before ){
		if ( $post_after->post_type === 'download' && $post_after->post_status === 'publish' && $post_before->post_status === 'publish' ){
			if ( $post_after->post_author !== $post_before->post_author ){
				$download     = new EDD_Download( $post_id );
				$sales_value  = $download->earnings;
				$sales_count  = $download->sales;
				if ( EDD_FES()->vendors->user_is_vendor( $post_before->post_author ) ){
					$vendor = new FES_Vendor( $post_before->post_author, true );
					// decrement sales
					$vendor->decrease_value( $sales_value );
					// decrement earnings
					$vendor->decrease_sales_count( $sales_count ); 
					// decrement products
					$vendor->decrease_product_count( 1 );
				}

				if ( EDD_FES()->vendors->user_is_vendor( $post_after->post_author  ) ){
					$vendor = new FES_Vendor( $post_after->post_author, true );
					// increment sales
					$vendor->increase_value( $sales_value );
					// increment earnings
					$vendor->increase_sales_count( $sales_count ); 
					// increment products
					$vendor->increase_product_count( 1 );
				}
			}
		}
	}

	public function post_status_transition( $new_status, $old_status, $post ){
		
		// Not an object if its not a draft yet. So prior to autosave this might throw warnings
		// We can prevent this by returning till it's been autosaved. This is when it becomes an obj.
		if ( ! is_object( $post ) ) {
			return;
		}


		if ( 'download' !== $post->post_type ){
	        return;
	    }

		$download     = new EDD_Download( $post->ID );
		$sales_value  = $download->earnings;
		$sales_count  = $download->sales;
		if ( EDD_FES()->vendors->user_is_vendor( $post->post_author ) ){
			if ( 'publish' !== $new_status && 'publish' === $old_status ){
				$vendor = new FES_Vendor( $post->post_author, true );
				// decrement sales
				$vendor->decrease_value( $sales_value );
				// decrement earnings
				$vendor->decrease_sales_count( $sales_count ); 
				// decrement products
				$vendor->decrease_product_count( 1 );
				return;
			}

			if ( 'publish' === $new_status && 'publish' !== $old_status ){
				$vendor = new FES_Vendor( $post->post_author, true );
				// increment sales
				$vendor->increase_value( $sales_value );
				// increment earnings
				$vendor->increase_sales_count( $sales_count ); 
				// increment products
				$vendor->increase_product_count( 1 );
				return;
			}
		}
	}

	// we need these to fire on the frontend
	public function increment_vendor_earnings( $payment_id ){

		$cart_details   = edd_get_payment_meta_cart_details( $payment_id );
		if ( is_array( $cart_details ) ) {
			foreach ( $cart_details as $cart_index => $download ) {
				// Increase the earnings for this download ID
				$post = get_post( $download['id'] );
				$author = $post->post_author;
				if ( EDD_FES()->vendors->user_is_vendor( $author ) ){
					$vendor = new FES_Vendor( $author, true );
					$vendor->increase_value( $download['price'] * $download['quantity'] );
					$vendor->increase_sales_count( $download['quantity'] );
				}
			}
		}
	}

	public function decrement_vendor_earnings( $payment_id ){

		$amount      = edd_get_payment_amount( $payment_id );
		$status      = $post->post_status;
		$customer_id = edd_get_payment_customer_id( $payment_id );
		if( $status == 'revoked' || $status == 'publish' ) {
			$cart_details   = edd_get_payment_meta_cart_details( $payment_id );
			foreach ( $cart_details as $cart_index => $download ) {
				// Increase the earnings for this download ID
				$post = get_post( $download['id'] );
				$author = $post->post_author;
				if ( EDD_FES()->vendors->user_is_vendor( $author ) ){
					$vendor = new FES_Vendor( $author, true );
					$vendor->decrease_value( $download['price'] * $download['quantity'] );
					$vendor->decrease_sales_count( $download['quantity']  );
				}
			}
		}
	}


	public function hide_admin_bar( $bool ) {

		if ( !fes_is_frontend() ) {
			return $bool;
		}

		// This setting is reversed. ! removed means it is removed. Stupid.
		if ( ! EDD_FES()->helper->get_option( 'fes-remove-admin-bar', false ) ) {
			if ( EDD_FES()->vendors->user_is_vendor() && ! (bool) EDD_FES()->vendors->user_is_admin() ) {
				$bool = false;
			}
		}

		return $bool;
	}
	public static function get_vendor_store_url_dashboard( $user = false ) {
		if ( is_numeric( $user ) ) {
			$user = new WP_User( $user );
		}
		if ( ! $user || ! is_object( $user ) ) {
			$user = new WP_User( get_current_user_id() );
		}
		$vendor_url = EDD_FES()->vendors->get_vendor_store_url( $user );
		return sprintf( __( ' Your store url is: %s', 'edd_fes' ), '<a href="' . esc_url( $vendor_url ) . '">' . $vendor_url . '</a>' );
	}

	public static function get_vendor_store_url( $user = false ) {

		if ( !is_object( $user ) ) {
			if ( $user === false ) {
				$user = get_current_user_id();
			}
			$user = new WP_User( $user );
		}

		if ( !$user || !is_object( $user ) ) {
			$user = new WP_User( get_current_user_id() );
		}

		$name = get_userdata( $user->ID );
		$user_nicename = apply_filters( 'fes_user_nicename_to_lower' , strtolower( $name->user_nicename ), $user );
		$vendor_url = get_permalink( EDD_FES()->helper->get_option( 'fes-vendor-page', '' ) );
		$permalink  = apply_filters( 'fes_adjust_vendor_url', untrailingslashit( 'vendor/' ) );
		$vendor_url = str_replace( 'fes-vendor/', $permalink, $vendor_url );
		$vendor_url = str_replace( 'vendor/', $permalink, $vendor_url );

		if ( get_option( 'permalink_structure' ) ) {
			$vendor_url = trailingslashit( $vendor_url ) . $user_nicename;
		} else {
			$vendor_url = add_query_arg( 'vendor', $user_nicename, $vendor_url );
		}

		return $vendor_url;
	}

	public function prevent_admin_access() {
		if ( EDD_FES()->helper->get_option( 'fes-allow-backend-access', false ) ) {
			return;
		}

		if (
			// Look for the presence of /wp-admin/ in the url
			stripos( $_SERVER['REQUEST_URI'], '/wp-admin/' ) !== false &&
			// Allow calls to async-upload.php
			stripos( $_SERVER['REQUEST_URI'], 'async-upload.php' ) == false &&
			// Allow calls to media-upload.php
			stripos( $_SERVER['REQUEST_URI'], 'media-upload.php' ) == false &&
			// Allow calls to admin-ajax.php
			stripos( $_SERVER['REQUEST_URI'], 'admin-ajax.php' ) == false ) {
			if ( EDD_FES()->vendors->user_is_vendor() && !EDD_FES()->vendors->user_is_admin() ) {
				wp_redirect( get_permalink( EDD_FES()->helper->get_option( 'fes-vendor-dashboard-page', get_permalink() ) ) );
				exit();
			}
		}
	}


    public function vendor_exists( $user = false, $by_user_id = true ){
		
		if ( $user === -2 ) {
			$user = get_current_user_id();
		}

		if ( $user == 0 ) {
			// This is a logged out user, since get_current_user_id returns 0 for non logged in
			// since we can't do anything with them, lets get them out of here. They aren't vendors.
			return false;
		}

		if ( is_numeric( $user ) ) {
			$field = $by_user_id ? 'user_id' : 'id';
		} else if ( is_email( $user ) ) {
			$field = 'email';
		} else {
			$field = 'username';
		}

		$db     = new FES_DB_Vendors;
		$vendor = $db->get_vendor_by( $field, $user );

		$output = true;
		if ( empty( $vendor ) || ! is_object( $vendor ) ) {
			$output = false;
		}
		$output = apply_filters( 'fes_vendors_vendor_exists', $output, $vendor, $user, $by_user_id );
		return $output;
    }

    public function user_is_vendor( $user = -2 ){
		if ( $user === -2 ) {
			$user = get_current_user_id();
		}

		if ( $user == 0 ) {
			// This is a logged out user, since get_current_user_id returns 0 for non logged in
			// since we can't do anything with them, lets get them out of here. They aren't vendors.
			return false;
		}


		if ( is_numeric( $user ) ) {
			$field = 'user_id';
		} else if ( is_email( $user ) ) {
			$field = 'email';
		} else {
			$field = 'username';
		}

		$db     = new FES_DB_Vendors;
		$vendor = $db->get_vendor_by( $field, $user );

		if ( empty( $vendor ) || ! is_object( $vendor ) ) {
			return false;
		} else {
			$output = false;
			if ( $vendor->status === 'approved' ) {
				$output = true;
			}
			$output = apply_filters( 'fes_vendors_user_is_vendor', $output, $vendor, $user );
			return $output;
		}
    }

    // warning: to use this function as named, the second param must be false ( backwards compat reasons for this )
    public function vendor_is_vendor( $user = -2, $by_user_id = true ){

		if ( $user === -2 ) {
			if ( $by_user_id ) { 
				$user = get_current_user_id();
			} else {
				$user = EDD_FES()->vendors->vendor_from_user( get_current_user_id() );
			}
		}

		if ( $user == 0 ) {
			// This is a logged out user, since get_current_user_id returns 0 for non logged in
			// since we can't do anything with them, lets get them out of here. They aren't vendors.
			return false;
		}


		if ( is_numeric( $user ) ) {
			$field = $by_user_id ? 'user_id' : 'id';
		} else if ( is_email( $user ) ) {
			$field = 'email';
		} else {
			$field = 'username';
		}

		$db     = new FES_DB_Vendors;
		$vendor = $db->get_vendor_by( $field, $user );

		if ( empty( $vendor ) || ! is_object( $vendor ) ) {
			return false;
		} else {
			$output = false;
			if ( $vendor->status === 'approved' ) {
				$output = true;
			}
			$output = apply_filters( 'fes_vendors_vendor_is_vendor', $output, $vendor, $user );
			return $output;
		}
    }

    public function user_from_vendor( $user = false ){
		
		if ( $user === -2 ) {
			$user = get_current_user_id();
		}

		if ( $user == 0 ) {
			// This is a logged out user, since get_current_user_id returns 0 for non logged in
			// since we can't do anything with them, lets get them out of here. They aren't vendors.
			return false;
		}

		if ( is_numeric( $user ) ) {
			$field = 'id';
		} else if ( is_email( $user ) ) {
			$field = 'email';
		} else {
			$field = 'username';
		}

		$db     = new FES_DB_Vendors;
		$vendor = $db->get_vendor_by( $field, $user );

		if ( empty( $vendor ) || ! is_object( $vendor ) ) {
			return false;
		} else {
			return $vendor->user_id;
		}

    }

     public function vendor_from_user( $user = false ){
		
		if ( $user === -2 ) {
			$user = EDD_FES()->vendors->vendor_from_user( get_current_user_id() );
		}

		if ( $user == 0 ) {
			// This is a logged out user, since get_current_user_id returns 0 for non logged in
			// since we can't do anything with them, lets get them out of here. They aren't vendors.
			return false;
		}

		if ( is_numeric( $user ) ) {
			$field = 'user_id';
		} else if ( is_email( $user ) ) {
			$field = 'email';
		} else {
			$field = 'username';
		}

		$db     = new FES_DB_Vendors;
		$vendor = $db->get_vendor_by( $field, $user );

		if ( empty( $vendor ) || ! is_object( $vendor ) ) {
			return false;
		} else {
			return $vendor->id;
		}

    }   


    public function user_is_status( $status = '', $user = -2  ){

		if ( $user === -2 ) {
			$user = get_current_user_id();
		}

		if ( $user == 0 ) {
			// This is a logged out user, since get_current_user_id returns 0 for non logged in
			// since we can't do anything with them, lets get them out of here. They aren't vendors.
			return false;
		}


		if ( is_numeric( $user ) ) {
			$field = 'user_id';
		} else if ( is_email( $user ) ) {
			$field = 'email';
		} else {
			$field = 'username';
		}

		$db     = new FES_DB_Vendors;
		$vendor = $db->get_vendor_by( $field, $user );

		if ( empty( $vendor ) || ! is_object( $vendor ) ) {
			return false;
		} else {
			$output = false;
			if ( $vendor->status === $status ) {
				$output = true;
			}
			$output = apply_filters( 'fes_vendors_user_is_status', $output, $status, $vendor, $user );
			return $output;
		}
    }


    public function vendor_is_status( $status = '', $user = -2 ){

		if ( $user === -2 ) {
			$user = EDD_FES()->vendors->vendor_from_user( get_current_user_id() );
		}

		if ( $user == 0 ) {
			// This is a logged out user, since get_current_user_id returns 0 for non logged in
			// since we can't do anything with them, lets get them out of here. They aren't vendors.
			return false;
		}


		if ( is_numeric( $user ) ) {
			$field = 'id';
		} else if ( is_email( $user ) ) {
			$field = 'email';
		} else {
			$field = 'username';
		}

		$db     = new FES_DB_Vendors;
		$vendor = $db->get_vendor_by( $field, $user );

		if ( empty( $vendor ) || ! is_object( $vendor ) ) {
			return false;
		} else {
			$output = false;
			if ( $vendor->status === $status ) {
				$output = true;
			}
			$output = apply_filters( 'fes_vendors_vendor_is_status', $output, $status, $vendor, $user );
			return $output;
		}
    }

    public function user_is_admin( $vendor = -2 ){

		if ( $vendor === -2 ) {
			$vendor = get_current_user_id();
		}

		if ( $vendor == 0 ) {
			// This is a logged out user, since get_current_user_id returns 0 for non logged in
			// since we can't do anything with them, lets get them out of here. They aren't vendors.
			return false;
		}

		$user = false;

		if ( is_numeric( $vendor ) ) {
			$user = new WP_User( $vendor );
		} else if ( is_email( $vendor ) ) {
			$user = get_user_by( 'email', $vendor );
		} else {
			$user = get_user_by( 'login', $vendor );
		}

		if ( !$user || ! is_object( $user ) ){
			return false; // user doesn't exist
		}		

		// This allows devs to take what would normally be a vendor and say they aren't a vendor.
		$bool = false;
		$bool = apply_filters( 'fes_skip_is_admin', $bool, $user );
		// Note to developers: I passed in the entire user object above.
		// So expect either an object (logged in user) or false (not logged in user).
		if ( $bool ) {
			return false;
		}
		// Authentication Attempt #1: okay let's try caps
		$vendor_caps = array( 'fes_is_admin', 'manage_options', 'manage_shop_settings', 'shop_worker', 'shop_accountant', 'shop_manager' );
		$vendor_caps = apply_filters( 'fes_is_admin_capabilities' , $vendor_caps );
		if ( is_array( $vendor_caps ) && !empty( $vendor_caps ) ){
			foreach( $vendor_caps as $vcap ){
				if ( user_can( $user->ID, $vcap ) ){
					return true;
				}
			}
		}

		// Authentication Attempt #2:  maybe a developer has a reason for wanting to hook a user in?
		$bool = false;
		$bool = apply_filters( 'fes_is_admin_check_override', $bool, $user );
		// Note to developers: I passed in the entire user object above.
		// So expect either an object (logged in user) or false (not logged in user).
		if ( $bool ) {
			return true;
		}
		// end of the line
		return false;
    }


    public function vendor_is_admin( $user = -2 ){

		if ( $user === -2 ) {
			$user = EDD_FES()->vendors->vendor_from_user( get_current_user_id() );
		}

		if ( $user == 0 ) {
			// This is a logged out user, since get_current_user_id returns 0 for non logged in
			// since we can't do anything with them, lets get them out of here. They aren't vendors.
			return false;
		}

    	$user = EDD_FES()->vendors->user_from_vendor( $user );
        return EDD_FES()->vendors->user_is_admin( $user );
    } 



	/**
	 * 
	 *
	 * @param unknown $input
	 * @return unknown
	 */
	public function get_user_id( $email_or_username = false ) {

		if ( $email_or_username === false ){
			return false;
		}

		if ( is_email( $email_or_username ) ) {
			$user = get_user_by( 'email', $email_or_username );
			if ( !$user || !is_object( $user ) ){
				return false;
			} else{
				return $user->ID;
			}
		} else {
			$user = get_user_by( 'login', $email_or_username );
			if ( !$user || !is_object( $user ) ){
				return false;
			} else{
				return $user->ID;
			}
		}
	}

	/**
	 * 
	 *
	 * @param unknown $input
	 * @return unknown
	 */
	public function get_vendor_id( $email_or_username = false  ) {
		
		if ( $email_or_username === false ){
			return false;
		}

		$user_id = false;

		if ( is_email( $email_or_username ) ) {
			$user = get_user_by( 'email', $email_or_username );
			if ( !$user || !is_object( $user ) ){
				return false;
			} else{
				$user_id = $user->ID;
			}
		} else {
			$user = get_user_by( 'login', $email_or_username );
			if ( !$user || !is_object( $user ) ){
				return false;
			} else{
				$user_id = $user->ID;
			}
		}

		return EDD_FES()->vendors->vendor_from_user( $user_id );
	}

	public function vendor_can_view_receipt( $user_can_view, $edd_receipt_id ) {
		if ( ! $edd_receipt_id ) {
			return false;
		}

		$payment_id = ! empty( $_GET['order_id'] ) ? absint( $_GET['order_id'] ) : false;

		if ( $payment_id ) {

			$cart = edd_get_payment_meta_cart_details( $payment_id );

			foreach ( $cart as $item ) {
				$item = get_post( $item['id'] );

				if ( $item->post_author == get_current_user_id() ) {
					$user_can_view = true;

					break;
				}
			}

		}

		return $user_can_view;
	}

	public function vendor_can_view_receipt_item( $user_can_view, $item ) {

		if ( is_user_logged_in() && !EDD_FES()->vendors->user_is_admin() && EDD_FES()->vendors->user_is_vendor( get_current_user_id() ) ) {

			$user_can_view = false;

			$download = get_post( $item['id'] );
			if ( (int) $download->post_author == (int) get_current_user_id() ) {
				$user_can_view = true;
			}

		}

		return $user_can_view;
	}

	public function vendor_is_author( $post_id = false, $user_id = false ) {
		if ( $post_id == false || $user_id == false ) {
			return false;
		}
		else {
			$download = get_post( $post_id ) ;
			if ( (int) $download->post_author !== (int) $user_id ) {
				return false;
			} else {
				return true;
			}
		}
	}

	public function vendor_can_create_product( $user_id = -2 ) {
		if ( $user_id == -2 ) {
			$user_id = get_current_user_id();
		}

		if ( !EDD_FES()->helper->get_option( 'fes-allow-vendors-to-create-products', true ) ) {
			return false;
		}

		if ( EDD_FES()->vendors->user_is_vendor( $user_id ) ) {
			return true;
		} else {
			return false;
		}
	}

	public function vendor_can_edit_product( $post_id ) {
		if ( !EDD_FES()->helper->get_option( 'fes-allow-vendors-to-edit-products', true ) ) {
			return false;
		}
		$user_id = get_current_user_id();
		$post = get_post( $post_id, ARRAY_A );
		if ( EDD_FES()->vendors->user_is_vendor( $user_id )  && ( EDD_FES()->vendors->user_is_admin( $user_id ) || $post['post_author'] == $user_id ) ) {
			return true;
		} else {
			return false;
		}
	}

	public function vendor_can_delete_product( $post_id ) {
		if ( !EDD_FES()->helper->get_option( 'fes-allow-vendors-to-delete-products', true ) ) {
			return false;
		}
		$user_id = get_current_user_id();
		$post = get_post( $post_id, ARRAY_A );
		if ( EDD_FES()->vendors->user_is_vendor( $user_id )  && ( EDD_FES()->vendors->user_is_admin( $user_id ) || $post['post_author'] == $user_id ) ) {
			return true;
		} else {
			return false;
		}
	}

	public function vendor_can_view_order( $post_id ) {
		if ( !EDD_FES()->helper->get_option( 'fes-allow-vendors-to-view-orders', false ) ) {
			return false;
		}
		$user_id = get_current_user_id();
		if ( EDD_FES()->vendors->user_is_vendor( $user_id ) || EDD_FES()->vendors->user_is_admin( $user_id ) ) {
			return edd_FES()->vendors->vendor_can_view_receipt( false, $post_id );
		} else {
			return false;
		}
	}

	public function vendor_can_view_orders() {
		if ( !EDD_FES()->helper->get_option( 'fes-allow-vendors-to-view-orders', false ) ) {
			return false;
		}
		$user_id = get_current_user_id();
		if ( EDD_FES()->vendors->user_is_vendor( $user_id ) || EDD_FES()->vendors->user_is_admin( $user_id ) ) {
			return true;
		} else {
			return false;
		}
	}

	public function get_pending_products( $user_id = false ) {
		global $wpdb;
		global $current_user;
		if ( !$user_id ) {
			$user_id = $user_id;
		}
		$vendor_products = array();
		$vendor_products = get_posts( array(
				'nopaging' => true,
				'author' => $user_id,
				'orderby' => 'title',
				'post_type' => 'download',
				'post_status' => 'pending',
				'order' => 'ASC'
			) );
		if ( empty( $vendor_products ) ) {
			return false;
		}
		foreach ( $vendor_products as $product ) {
			$data[] = array(
				'ID' => $product->ID,
				'date' => $product->post_date,
				'title' => $product->post_title,
				'url' => esc_url( get_permalink( $product->ID ) ),
				'sales' => edd_get_download_sales_stats( $product->ID )
			);
		}
		$data = apply_filters( 'fes_get_pending_products', $data, $user_id );
		return $data;
	}

	public function get_published_products( $user_id = false ) {
		global $wpdb;
		global $current_user;
		if ( !$user_id ) {
			$user_id = $current_user->ID;
		}
		$vendor_products = array();
		$vendor_products = get_posts( array(
				'nopaging' => true,
				'author' => $user_id,
				'orderby' => 'title',
				'post_type' => 'download',
				'post_status' => 'publish',
				'order' => 'ASC'
			) );
		if ( empty( $vendor_products ) ) {
			return false;
		}
		foreach ( $vendor_products as $product ) {
			$data[] = array(
				'ID' => $product->ID,
				'date' => $product->post_date,
				'title' => $product->post_title,
				'url' => esc_url( get_permalink( $product->ID ) ),
				'sales' => edd_get_download_sales_stats( $product->ID )
			);
		}
		$data = apply_filters( 'fes_get_published_products', $data, $user_id );
		return $data;
	}
	public function get_all_products( $user_id = false, $status = array( 'draft', 'pending', 'publish', 'trash', 'future', 'private' ) ) {
		global $wpdb;
		global $current_user;
		if ( !$user_id ) {
			$user_id = $current_user->ID;
		}
		$vendor_products = array();
		$vendor_products = get_posts( array(
				'nopaging' => true,
				'author' => $user_id,
				'orderby' => 'title',
				'post_type' => 'download',
				'post_status' => $status,
				'order' => 'ASC'
			) );
		if ( empty( $vendor_products ) ) {
			return false;
		}
		foreach ( $vendor_products as $product ) {
			$data[] = array(
				'ID' => $product->ID,
				'title' => $product->post_title,
				'status' => $product->post_status,
				'url' => esc_url( admin_url( 'post.php?post='.$product->ID.'&action=edit' ) ),
				'sales' => edd_get_download_sales_stats( $product->ID )
			);
		}
		$data = $this->array_msort( $data, array( 'status'=>SORT_ASC, 'title'=> SORT_ASC, 'sales' => SORT_DESC, 'ID' => SORT_ASC, 'url'=> SORT_ASC ) );
		$data = apply_filters( 'fes_get_all_products', $data, $user_id );
		return $data;
	}

	function array_msort( $array, $cols ) {
		$colarr = array();
		foreach ( $cols as $col => $order ) {
			$colarr[$col] = array();
			foreach ( $array as $k => $row ) { $colarr[$col]['_'.$k] = strtolower( $row[$col] ); }
		}
		$eval = 'array_multisort(';
		foreach ( $cols as $col => $order ) {
			$eval .= '$colarr[\''.$col.'\'],'.$order.',';
		}
		$eval = substr( $eval, 0, -1 ).');';
		eval( $eval );
		$ret = array();
		foreach ( $colarr as $col => $arr ) {
			foreach ( $arr as $k => $v ) {
				$k = substr( $k, 1 );
				if ( !isset( $ret[$k] ) ) $ret[$k] = $array[$k];
				$ret[$k][$col] = $array[$k][$col];
			}
		}
		return $ret;

	}

	public function get_products( $user_id, $post_status = false ) {
		global $wpdb;

		$products = array();

		$paged = ( get_query_var( 'paged' ) ) ? get_query_var( 'paged' ) : 1;

		$allowed_statuses = array( 'publish', 'draft', 'pending', 'future' );

		if ( empty( $post_status ) || ! in_array( $post_status, $allowed_statuses ) ) {
			$post_status = array( 'publish', 'draft', 'pending', 'future' );
		}

		$args = array(
			'author' => $user_id,
			'post_type' => 'download',
			'post_status' => $post_status,
			'posts_per_page' => 10,
			'orderby' => 'post_date',
			'order' => 'DESC',
			'paged' => $paged
		);

		$args = apply_filters( 'fes_get_products_args', $args );

		$products = get_posts( $args );

		if ( empty( $products ) ) {
			return array();
		}

		$data = apply_filters( 'fes_get_products_data', $products );
		return $data;
	}

	public function get_all_products_count( $user_id, $status ) {
		$products = $this->get_all_products( $user_id, $status );

		return !empty( $products ) ? count( $products ) : 0;
	}

	public function get_all_orders( $user_id = 0, $status = array() ) {
		$published_products = EDD_FES()->vendors->get_published_products( $user_id );
		if ( !$published_products ) {
			return array();
		}
		$published_products = wp_list_pluck( $published_products, 'ID' );

		$paged = ( get_query_var( 'paged' ) ) ? get_query_var( 'paged' ) : 1;

		$args = array(
			'download' => $published_products,
			'output'   => 'edd_payment',
			'mode'     => 'all',
			'number' => 10,
			'orderby' => 'post_date',
			'order' => 'DESC',
			'paged' => $paged
		);
		$args = apply_filters( 'fes_get_all_orders_args', $args );

		$payments = edd_get_payments( $args );

		if ( ! $payments ) {
			return array();
		}

		// nothing fancy with this for now
		return $payments;
	}

	public function get_all_orders_count( $user_id, $status ) {
		return count( $this->get_all_orders( $user_id, $status ) );
	}

	public function can_see_login() {
		if ( !is_user_logged_in() ) {
			return true;
		} else {
			return false;
		}
	}

	public function can_see_registration() {
		if ( !EDD_FES()->vendors->user_is_status( 'pending' ) && !EDD_FES()->vendors->user_is_vendor() && ( EDD_FES()->helper->get_option( 'fes-allow-registrations', true ) || EDD_FES()->helper->get_option( 'fes-allow-applications', true ) )  ) {
			return true;
		} else {
			return false;
		}

	}


	public function can_get_form_values( $id = -2, $is_admin = -2, $user_id = -2, $public = -2 ) {
		return true;
	}

	public function combo_form_count() {
		if ( EDD_FES()->vendors->can_see_registration() && EDD_FES()->vendors->can_see_login() ) {
			return 2;
		} else if ( EDD_FES()->vendors->can_see_registration() || EDD_FES()->vendors->can_see_login() ) {
			return 1;
		} else {
			return 0;
		}
	}

	public function get_avatar( $hook_name = '', $id_or_email = 0 , $size = 96 , $default = '' , $alt = false ) {
		if ( $id_or_email == 0 ) {
			$id_or_email = get_current_user_id();
		}

		$id_or_email = apply_filters( 'fes_get_avatar_id_or_email_' . $hook_name , $id_or_email );
		$size        = apply_filters( 'fes_get_avatar_size_' . $hook_name , $size );
		$default     = apply_filters( 'fes_get_avatar_id_or_email_' . $hook_name , $default );
		$alt         = apply_filters( 'fes_get_avatar_alt_' . $hook_name , $alt );
		$avatar      = apply_filters( 'fes_get_avatar', get_avatar( $id_or_email, $size, $default, $alt ) );

		return apply_filters( 'fes_get_avatar_' . $hook_name, $avatar, $id_or_email, $size, $default, $alt );

	}

	public function can_save_formbuilder( $post_id = -2 ) {
		if ( $post_id === -2 ) {
			$post_id = get_the_ID();
		}

		// Is the user allowed to see the formbuilder?
		if ( !current_user_can( 'edit_post', $post_id ) ) {
			return false;
		} else {
			return apply_filters( 'fes_can_save_formbuilder', true, $post_id );
		}
	}

    public function make_user_vendor( $user_id = -2 ){
        if ( $user_id === -2 ){
            $user_id = get_current_user_id();
        }

        $db_user     = new FES_DB_Vendors();

        if ( $db_user->exists( 'id', $user_id ) ){
            return;
        }

        // create user
        $user = new WP_User( $user_id );
            // note: Insert as pending then set to approved right underneath.
        $db_user->add( array(
            'user_id'        => $user->ID,
            'email'          => $user->user_email,
            'username'       => $user->user_login,
            'name'           => $user->display_name,
            'product_count'  => 0,
            'sales_count'    => 0,
            'sales_value'    => 0.00,
            'status'         => 'pending',
            'notes'          => '',
            'date_created'   => date( 'Y-m-d H:i:s' ),
        ) );

        // set to approved
        $vendor  = new FES_Vendor( $user->ID, true );
        $vendor->change_status( 'approved', false );
    }	

	/**
	 * FES Deprecated Functions:
	 * All functions below this point deprecated in 2.3
	 * and slated for removal in 2.4
	 */


	// This deprecated function will be removed in 2.4
	public static function is_pending( $user_id = -2 ) {
		_fes_deprecated_function( 'EDD_FES()->vendors->is_vendor', '2.3', 'EDD_FES()->vendors->user_is_status' );
		if ( $user_id == -2 ) {
			$user_id = get_current_user_id();
		}
		$user       = get_userdata( $user_id );
		$roles      = ! empty( $user->roles ) ? (array) $user->roles : array();
		$is_pending = in_array( 'pending_vendor', $roles );
		return $is_pending;
	}

	// This deprecated function will be removed in 2.4
	public static function is_frontend( $user_id = -2 ) {
		_fes_deprecated_function( 'EDD_FES()->vendors->is_vendor', '2.3', 'EDD_FES()->vendors->user_is_status' );
		if ( $user_id == -2 ) {
			$user_id = get_current_user_id();
		}
		$user       = get_userdata( $user_id );
		$roles      = ! empty( $user->roles ) ? (array) $user->roles : array();
		$is_frontend = in_array( 'frontend_vendor', $roles );
		return $is_frontend;
	}

	// This deprecated function will be removed in 2.4
	public static function is_suspended( $user_id = -2 ) {
		_fes_deprecated_function( 'EDD_FES()->vendors->is_vendor', '2.3', 'EDD_FES()->vendors->user_is_status' );
		if ( $user_id == -2 ) {
			$user_id = get_current_user_id();
		}
		$user       = get_userdata( $user_id );
		$roles      = ! empty( $user->roles ) ? (array) $user->roles : array();
		$is_suspended = in_array( 'suspended_vendor', $roles );
		return $is_suspended;
	}

	// This deprecated function will be removed in 2.4
	public static function is_vendor( $user_id ) {
		_fes_deprecated_function( 'EDD_FES()->vendors->is_vendor', '2.3', 'EDD_FES()->user_is_vendor' );
		$bool      = user_can( 'frontend_vendor', $user_id ) ? true : false;
		return apply_filters( 'fes_is_vendor', $bool, $user_id );
	}

	// This deprecated function will be removed in 2.4
	public static function is_admin( $user_id ) {
		_fes_deprecated_function( 'EDD_FES()->vendors->is_vendor', '2.3', 'EDD_FES()->user_is_admin' );
		$bool      = user_can( 'fes_is_admin', $user_id ) ? true : false;
		return apply_filters( 'fes_is_admin', $bool, $user_id );
	}

	// This deprecated function will be removed in 2.4
	public function get_product_constant_name( $plural = false, $uppercase = true ) {
		_fes_deprecated_function( 'EDD_FES()->vendors->get_product_constant_name()', '2.3', 'EDD_FES()->helper->get_product_constant_name()' );
		return EDD_FES()->helper->get_product_constant_name( $plural, $uppercase );
	}

	// This deprecated function will be removed in 2.4
	public function get_vendor_constant_name( $plural = false, $uppercase = true ) {
		_fes_deprecated_function( 'EDD_FES()->vendors->get_vendor_constant_name()', '2.3', 'EDD_FES()->helper->get_vendor_constant_name()' );
		return EDD_FES()->helper->get_vendor_constant_name( $plural, $uppercase );
	}	
}