<?php
/**
 * Plugin Name:         Easy Digital Downloads - Frontend Submissions
 * Plugin URI:          https://easydigitaldownloads.com/extensions/frontend-submissions/
 * Description:         Mimick Etsy, Envato, or Amazon type sites with this plugin and Easy Digital Downloads combined!
 * Author:              Chris Christoff
 * Author URI:          http://www.chriscct7.com
 *
 * Version:             2.3.5
 * Requires at least:   4.2
 * Tested up to:        4.4
 *
 * Text Domain:         edd_fes
 * Domain Path:         /languages/
 *
 * @category            Plugin
 * @copyright           Copyright © 2015 Chris Christoff
 * @author              Chris Christoff
 * @package             FES
 */

if ( !defined( 'ABSPATH' ) ) {
	exit;
}

/** Check if Easy Digital Downloads is active */
include_once( ABSPATH . 'wp-admin/includes/plugin.php' );

class EDD_Front_End_Submissions {
	/**
	 * @var EDD_Front_End_Submissions The one true EDD_Front_End_Submissions
	 * @since 1.4
	 */
	private static $instance;
	public $id = 'edd_fes';
	public $basename;

	// Setup objects for each class
	public $forms;
	public $templates;
	public $setup;
	public $emails;
	public $vendors;
	public $vendor_permissions;
	public $vendor_shop;
	public $dashboard;
	public $queries;
	public $menu;
	public $comments;
	public $helper;
	public $download_table;
	public $edit_download;
	public $edit_vendor;
	public $load_forms;
	public $load_fields;
	//public $formbuilder; // for 2.4
	public $formbuilder_templates;

	public $fes_options; // Here for backwards compatibility

	/**
	 * Main EDD_Front_End_Submissions Instance
	 *
	 * Insures that only one instance of EDD_Front_End_Submissions exists in memory at any one
	 * time. Also prevents needing to define globals all over the place.
	 *
	 * @since 2.0
	 * @static
	 * @staticvar array $instance
	 * @uses EDD_Front_End_Submissions::setup_globals() Setup the globals needed
	 * @uses EDD_Front_End_Submissions::includes() Include the required files
	 * @uses EDD_Front_End_Submissions::setup_actions() Setup the hooks and actions
	 * @see EDD()
	 * @return The one true EDD_Front_End_Submissions
	 */
	public static function instance() {
		global $wp_version;

		if ( version_compare( $wp_version, '4.2', '<' ) ) {
			add_action( 'admin_notices', array('EDD_Front_End_Submissions','wp_notice' ) );
			return;

		} else if ( !class_exists( 'Easy_Digital_Downloads' ) || version_compare( EDD_VERSION, '2.3', '<' ) ) {
			add_action( 'admin_notices', array('EDD_Front_End_Submissions','edd_notice' ) );
			return;
		}

		if ( !isset( self::$instance ) && !( self::$instance instanceof EDD_Front_End_Submissions ) ) {
			self::$instance = new EDD_Front_End_Submissions;
			self::$instance->define_globals();
			self::$instance->includes();
			
			$fes_version = get_option( 'fes_current_version', '2.1' );

			if ( version_compare( $fes_version, '2.3.2', '<' ) ) {
				fes_call_install();
			}

			self::$instance->setup();

			add_action( 'plugins_loaded', array( self::$instance, 'load_forms_and_fields' ) );

			// Setup class instances
			self::$instance->helper 			   = new FES_Helpers;
			self::$instance->emails                = new FES_Emails;
			self::$instance->vendors               = new FES_Vendors;
			self::$instance->integrations		   = new FES_Integrations;
			self::$instance->fes_options		   = self::$instance->helper; // Backwards compatibility

			if ( fes_is_admin() ) {
				self::$instance->menu                  = new FES_Menu;
				self::$instance->download_table        = new FES_Download_Table;
				self::$instance->edit_download         = new FES_Edit_Download;
				//self::$instance->formbuilder           = new FES_Formbuilder; // for 2.4
				self::$instance->formbuilder_templates = new FES_Formbuilder_Templates;
			}

			self::$instance->templates             = new FES_Templates;
			self::$instance->vendor_shop           = new FES_Vendor_Shop;
			self::$instance->dashboard             = new FES_Dashboard;
			self::$instance->forms		           = new FES_Forms;

			require_once EDD_PLUGIN_DIR . 'includes/admin/upload-functions.php';
			require_once EDD_PLUGIN_DIR . 'includes/misc-functions.php';
		}
		return self::$instance;
	}
	
	/**
	 * Throw error on object clone
	 *
	 * The whole idea of the singleton design pattern is that there is a single
	 * object therefore, we don't want the object to be cloned.
	 *
	 * @since 2.3
	 * @access protected
	 * @return void
	 */
	public function __clone() {
		// Cloning instances of the class is forbidden
		_doing_it_wrong( __FUNCTION__, __( 'Cheatin&#8217; huh?', 'edd_fes' ), '2.3' );
	}

	/**
	 * Disable unserializing of the class
	 *
	 * @since 2.3
	 * @access protected
	 * @return void
	 */
	public function __wakeup() {
		// Unserializing instances of the class is forbidden
		_doing_it_wrong( __FUNCTION__, __( 'Cheatin&#8217; huh?', 'edd_fes' ), '2.3' );
	}

	public function define_globals() {
		$this->title    = __( 'Frontend Submissions', 'edd_fes' );
		$this->file     = __FILE__;
		$this->basename = apply_filters( 'fes_plugin_basename', plugin_basename( $this->file ) );
		// Plugin Name
		if ( !defined( 'fes_plugin_name' ) ) {
			define( 'fes_plugin_name', 'Frontend Submissions' );
		}
		// Plugin Version
		if ( !defined( 'fes_plugin_version' ) ) {
			define( 'fes_plugin_version', '2.3.5' );
		}
		// Plugin Root File
		if ( !defined( 'fes_plugin_file' ) ) {
			define( 'fes_plugin_file', __FILE__ );
		}
		// Plugin Folder Path
		if ( !defined( 'fes_plugin_dir' ) ) {
			define( 'fes_plugin_dir', WP_PLUGIN_DIR . '/' . basename( dirname( __FILE__ ) ) . '/' );
		}
		// Plugin Folder URL
		if ( !defined( 'fes_plugin_url' ) ) {
			define( 'fes_plugin_url', plugin_dir_url( fes_plugin_file ) );
		}
		// Plugin Assets URL
		if ( !defined( 'fes_assets_url' ) ) {
			define( 'fes_assets_url', fes_plugin_url . 'assets/' );
		}
	}

	public function load_forms_and_fields(){
		// load form abstract and extending forms
		self::$instance->load_forms            = self::$instance->setup->load_forms();
		
		// load field abstract and extending fields
		self::$instance->load_fields           = self::$instance->setup->load_fields();
	}

	public function includes() {
		require_once fes_plugin_dir . 'classes/class-helpers.php';
		require_once fes_plugin_dir . 'classes/class-vendor.php';
		require_once fes_plugin_dir . 'classes/class-db-vendor.php';
		require_once fes_plugin_dir . 'classes/class-vendors.php';
		require_once fes_plugin_dir . 'classes/class-emails.php';
		require_once fes_plugin_dir . 'classes/class-integrations.php';
		require_once fes_plugin_dir . 'classes/misc-functions.php';
		if ( !class_exists( 'WP_List_Table' ) ) {
			require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
		}
		require_once fes_plugin_dir . 'classes/admin/class-update.php';

		if ( fes_is_admin() ){
			require_once fes_plugin_dir . 'classes/schema.php';
			require_once fes_plugin_dir . 'classes/admin/vendors/vendors.php';
			require_once fes_plugin_dir . 'classes/admin/vendors/vendor-actions.php';
			require_once fes_plugin_dir . 'classes/admin/vendors/vendor-functions.php';
			require_once fes_plugin_dir . 'classes/admin/vendors/graphing.php';
			require_once fes_plugin_dir . 'classes/admin/vendors/class-export-customers.php';
			require_once fes_plugin_dir . 'classes/admin/vendors/pdf-reports.php';
			require_once fes_plugin_dir . 'classes/admin/class-tracking.php';
			require_once fes_plugin_dir . 'classes/admin/class-welcome.php';
			require_once fes_plugin_dir . 'classes/admin/class-tools.php';
			require_once fes_plugin_dir . 'classes/admin/class-menu.php';
			require_once fes_plugin_dir . 'classes/admin/class-list-table.php';
			require_once fes_plugin_dir . 'classes/admin/downloads/class-download-table.php';
			require_once fes_plugin_dir . 'classes/admin/downloads/class-edit-download.php';
			require_once fes_plugin_dir . 'classes/admin/vendors/class-vendor-table.php';
			require_once fes_plugin_dir . 'classes/admin/formbuilder/class-formbuilder.php';
			require_once fes_plugin_dir . 'classes/admin/formbuilder/class-formbuilder-templates.php';
		}
		
		require_once fes_plugin_dir . 'classes/frontend/class-vendor-shop.php';
		require_once fes_plugin_dir . 'classes/frontend/class-templates.php';
		require_once fes_plugin_dir . 'classes/frontend/class-dashboard.php';
		require_once fes_plugin_dir . 'classes/frontend/class-forms.php';			

		add_filter( 'edd_template_paths', array( $this, 'edd_template_paths' ) );
	}

	public function setup() {
		require_once fes_plugin_dir . 'classes/class-post-types.php';
		require_once fes_plugin_dir . 'classes/class-setup.php';

		// load textdomains
		$this->load_textdomain();

		// load settings
		$this->load_settings();
		self::$instance->setup = $this->setup = new FES_Setup;

		if ( class_exists( 'EDD_License' ) ) {
			$license = new EDD_License( __FILE__, fes_plugin_name, fes_plugin_version, 'Chris Christoff' );
		}
	}

	public function load_textdomain() {
		$locale        = apply_filters( 'plugin_locale', 'en_US', 'edd_fes' );
		$mofile        = sprintf( '%1$s-%2$s.mo', 'edd_fes', $locale );

		$mofile_local  = trailingslashit( fes_plugin_dir . 'languages' ) . $mofile;
		$mofile_global = WP_LANG_DIR . '/edd_fes/' . $mofile;

		if ( file_exists( $mofile_global ) ) {
			return load_textdomain( 'edd_fes', $mofile_global );
		} else if ( file_exists( $mofile_local ) ) {
			return load_textdomain( 'edd_fes', $mofile_local );
		} else{
			load_plugin_textdomain( 'edd_fes', false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );
		}
	}

	public function edd_template_paths( $paths ) {
		$paths[80] = trailingslashit( fes_plugin_dir ) . trailingslashit( 'templates' );

		return $paths;
	}

	public function load_settings() {
		if ( !class_exists( 'ReduxFramework' ) && file_exists( dirname( __FILE__ ) . '/classes/redux/ReduxCore/framework.php' ) ) {
			require_once( dirname( __FILE__ ) . '/classes/redux/ReduxCore/framework.php' );
		}
		require_once( dirname( __FILE__ ) . '/classes/admin/class-settings.php' );
	}

	public static function edd_notice() { ?>
		<div class="updated">
			<p><?php printf( __( '<strong>Notice:</strong> Easy Digital Downloads Frontend Submissions requires Easy Digital Downloads 2.3 or higher in order to function properly.', 'edd_fes' ) ); ?></p>
		</div>
		<?php
	}
	public static function wp_notice() { ?>
		<div class="updated">
			<p><?php printf( __( '<strong>Notice:</strong> Easy Digital Downloads Frontend Submissions requires WordPress 4.2 or higher in order to function properly.', 'edd_fes' ) ); ?></p>
		</div>
		<?php
	}
}

/**
 * The main function responsible for returning the one true EDD_Front_End_Submissions
 * Instance to functions everywhere.
 *
 * Use this function like you would a global variable, except without needing
 * to declare the global.
 *
 * Example: <?php $edd_fes = EDD_FES(); ?>
 *
 * @since 2.0
 * @return object The one true EDD_Front_End_Submissions Instance
 */
function EDD_FES() {
	return EDD_Front_End_Submissions::instance();
}

EDD_FES();

function FES_Install() {
	global $wp_version;
	if ( version_compare( $wp_version, '4.2', '<' ) ) {
		if ( is_plugin_active( EDD_FES()->basename ) ) {
			return;
		}
	} else if ( !class_exists( 'Easy_Digital_Downloads' ) || version_compare( EDD_VERSION, '2.3', '<' ) ) {
		if ( is_plugin_active( EDD_FES()->basename ) ) {
			return;
		}
	}
	EDD_FES()->load_settings();
	require_once fes_plugin_dir . 'classes/schema.php';
	require_once fes_plugin_dir . 'classes/class-post-types.php';
	require_once fes_plugin_dir . 'classes/admin/class-install.php';
	$install = new FES_Install;
	$install->init();
}

function fes_call_install(){
	add_action( 'wp_loaded', 'FES_Install' );
}
